"""Module for processing image files."""
import matplotlib.image as mpimg
from scipy import fftpack, ndimage
import numpy as np
from matplotlib import pyplot as plt

# from scipy.ndimage import rotate
import scipy


class Image(object):
    """Module for processing image files."""

    def __init__(self, values=[], pbc_params=[]):
        """Initialize the module."""
        # pbc_params for 2D image: a,b and gamma
        # 3D : a,b,c, alpha, beta, gamma

        self.values = values  # XY numpy array
        self.pbc_params = pbc_params

    def zoom_interp_2d(self, FFT_image, zoom_factor=40, interpol_factor=1):
        """Zoom and interpolate an image."""
        zoom_size = (FFT_image.shape[0] / zoom_factor) / 2
        window_size_x = FFT_image.shape[0]
        window_size_y = FFT_image.shape[1]

        if np.mod(FFT_image.shape[0] / zoom_factor, 2) == 0:
            F2_zoomed = FFT_image[
                int(window_size_x / 2 - zoom_size) : int(
                    window_size_x / 2 + zoom_size
                ),
                int(window_size_y / 2 - zoom_size) : int(
                    window_size_y / 2 + zoom_size
                ),
            ]
        else:
            F2_zoomed = FFT_image[
                int(window_size_x / 2 - zoom_size) : int(
                    window_size_x / 2 + 1 + zoom_size
                ),
                int(window_size_y / 2 - zoom_size) : int(
                    window_size_y / 2 + 1 + zoom_size
                ),
            ]

        return ndimage.zoom(F2_zoomed, interpol_factor)

    def fourier_transform2D(
        self,
        zoom_factor=30,
        interpol_factor=1,
        use_crop=True,
        pad_bright_spot=True,
    ):
        """Make 2D FT."""
        if use_crop:

            g1 = fftpack.fft2(
                (self.crop_square().rgb_to_gray().values[:, :, 0])
            )
        else:
            g1 = fftpack.fft2((self.rgb_to_gray().values[:, :, 0]))
        g2 = np.abs((fftpack.fftshift((g1))))
        g2 = np.abs((fftpack.fftshift((g1))))
        g3 = self.zoom_interp_2d(
            g2, zoom_factor=zoom_factor, interpol_factor=interpol_factor
        )
        if pad_bright_spot:

            g3 = g3 / np.max(g3)
            x = int(g3.shape[0] / 2)
            y = int(g3.shape[1] / 2)
            g3[x, y] = 0.15

        print("g3 shape", x, y)
        # Zoom in to see the FFT more clearly.

        return Image(values=g3)
        # ft_arr=
        # return ft_arr

    @staticmethod
    def from_file(path):
        """Load image frim file."""
        im = mpimg.imread(path)
        return Image(values=im)
        pass

    def rgb_to_gray(self):
        """Make RGB to Grey scale image."""
        img = np.array(self.values)
        # grayImage = np.zeros(img.shape)
        R = np.array(img[:, :, 0])
        G = np.array(img[:, :, 1])
        B = np.array(img[:, :, 2])

        R = R * 0.299
        G = G * 0.587
        B = B * 0.114

        Avg = R + G + B

        for i in range(3):
            img[:, :, i] = Avg

        return Image(values=img)

    def crop_square(self, size=None):
        """Crop an image."""
        if size is None:
            size = min(self.values.shape[0], self.values.shape[1])
        img_cropped = self.values.copy()[:size, :size, :]

        return Image(values=img_cropped)

    def save(self, filename):
        """Save an image."""
        # if size is None:
        # from matplotlib import pyplot as plt

        plt.imshow(self.values, interpolation="nearest")
        plt.savefig(filename)
        plt.close()

    def black_and_white(self, threshold=127):
        """Make black and white image."""
        bw = np.asarray(self.values).copy()
        bw[bw < threshold] = 0
        bw[bw >= threshold] = 255
        return Image(values=bw)

    def gaussian_filter(self, sigma=10):
        """Apply Gaussian filter to an image."""
        sigmax = sigma
        sigmay = sigma

        cy, cx = int(self.values.shape[0] / 2), int(self.values.shape[1] / 2)
        x = np.linspace(0, self.values.shape[0], self.values.shape[0])
        y = np.linspace(0, self.values.shape[1], self.values.shape[1])
        X, Y = np.meshgrid(x, y)
        gmask = np.exp(-(((X - cx) / sigmax) ** 2 + ((Y - cy) / sigmay) ** 2))
        new_values = self.values * gmask
        return Image(values=new_values)

    def rotate(self, angle=45):
        """Rotate an image."""
        rot = scipy.ndimage.rotate(self.values, angle)
        return Image(values=rot)


"""
if __name__ == "__main__":
    p = "JVASP-667_neg.jpg"
    im = Image.from_file(p)

    plt.imshow(
        im.fourier_transform2D(use_crop=True, zoom_factor=50)
        .rotate(angle=0)
        .black_and_white(threshold=0.05)
        .values,
        cmap="Greys",
    )
"""



"""Module to generate XML file for LAMMPS calculation."""
import numpy as np
from jarvis.core.atoms import get_supercell_dims
from jarvis.db.jsonutils import loadjson
from jarvis.analysis.structure.spacegroup import Spacegroup3D
from jarvis.core.utils import stringdict_to_xml

mp_jv = loadjson("/rk2/knc6/DB/MP/mp_jv_id.json")


def get_jvid(mp=""):
    """Get JARVIS-ID for MPID."""
    jvid = ""
    try:
        jvid = "'" + ",".join(mp_jv[mp]) + "'"
    except Exception as exp:
        print("No JID", exp)
        pass
    return jvid


def basic_data(data={}, source="JARVIS-FF-LAMMPS"):
    """Get basic data for table."""
    info = {}
    info["id"] = data["jid"]
    info["source_folder"] = data["source_folder"]
    info["tmp_source_folder"] = "'" + data["source_folder"] + "'"
    info["tmp_id"] = "'" + data["jid"] + "'"
    ref = data["source_folder"].split("/")[-1].split("@")[1].split("_")[0]
    info["ref"] = "'" + ref + "'"
    info["jvid"] = get_jvid(ref)
    final_atoms = data["bulk_data"]["final_str"]
    initial_atoms = data["bulk_data"]["initial_str"]
    info["formula"] = final_atoms.composition.reduced_formula
    info["tmp_formula"] = "'" + final_atoms.composition.reduced_formula + "'"
    info["elements"] = ",".join(final_atoms.uniq_species)
    info["tmp_elements"] = "'" + ",".join(final_atoms.uniq_species) + "'"
    info["number_uniq_species"] = len(final_atoms.uniq_species)
    info["data_source"] = source
    info["tmp_data_source"] = "'" + source + "'"
    info["pair_style"] = data["bulk_data"]["pair_style"]
    info["pair_coeff"] = data["bulk_data"]["pair_coeff"]
    # info["pair_coeff"] = (
    #    '<a href="http://www.ctcms.nist.gov/~knc6/DOWNLOADS/'
    #    + data["bulk_data"]["pair_coeff"]
    #    + '.zip">'
    #    + data["bulk_data"]["pair_coeff"]
    #    + "</a>"
    # )
    info["energy_per_atom"] = round(
        float(data["bulk_data"]["energy_per_atom"]), 3
    )
    info["pressure"] = round(float(data["bulk_data"]["system_pressure"]), 3)

    initial_spg = Spacegroup3D(initial_atoms)
    final_spg = Spacegroup3D(final_atoms)
    info["initial_spacegroup_number"] = initial_spg.space_group_number
    info["initial_spacegroup_symbol"] = initial_spg.space_group_symbol
    info["initial_pointgroup_symbol"] = initial_spg.point_group_symbol
    info["initial_crystal_system"] = initial_spg.crystal_system
    initial_lat_params = initial_atoms.lattice.parameters
    info["initial_a"] = round(initial_lat_params[0], 2)
    info["initial_b"] = round(initial_lat_params[1], 2)
    info["initial_c"] = round(initial_lat_params[2], 2)
    info["initial_alpha"] = round(initial_lat_params[3], 2)
    info["initial_beta"] = round(initial_lat_params[4], 2)
    info["initial_gamma"] = round(initial_lat_params[5], 2)
    info["initial_density"] = round(initial_atoms.density, 3)
    dim = get_supercell_dims(final_atoms)
    info["xyz"] = (
        '"'
        + str(final_atoms.make_supercell_matrix(dim).get_xyz_string).replace(
            "\n", "\\n"
        )
        + '"'
    )
    info["final_str"] = (
        '"' + str(final_atoms.get_string()).replace("\n", "\\n") + '"'
    )
    info["initial_str"] = (
        '"' + str(initial_atoms.get_string()).replace("\n", "\\n") + '"'
    )
    final_lat_params = final_atoms.lattice.parameters
    info["final_a"] = round(final_lat_params[0], 2)
    info["final_b"] = round(final_lat_params[1], 2)
    info["final_c"] = round(final_lat_params[2], 2)
    info["final_alpha"] = round(final_lat_params[3], 2)
    info["final_beta"] = round(final_lat_params[4], 2)
    info["final_gamma"] = round(final_lat_params[5], 2)
    info["final_density"] = round(final_atoms.density, 3)

    info["final_spacegroup_number"] = final_spg.space_group_number
    info["final_spacegroup_symbol"] = final_spg.space_group_symbol
    info["final_pointgroup_symbol"] = final_spg.point_group_symbol
    info["final_crystal_system"] = final_spg.crystal_system

    et = ""
    # print(data["bulk_data"]["elastic_tensor"]["raw_et_tensor"])
    try:
        if data["bulk_data"]["elastic_tensor"] != "":
            cij = np.round(
                (data["bulk_data"]["elastic_tensor"]["raw_et_tensor"]), 2
            )
            et = (
                '<cij>"'
                + ";".join(
                    [",".join(map(str, cij[:, i])) for i in range(0, 6)]
                )
                + '"</cij>'
            )

    except Exception as exp:
        print("Cannot obtain elastic tensor data.", info["source_folder"], exp)
        pass

    info["elastic_tensor"] = et
    vacancy_line = ""
    if len(data["vacancy_info"]) != 0:
        for i in data["vacancy_info"]:
            vacancy_line += i[0] + "," + i[1] + "," + str(round(i[2], 3)) + ";"
    info["vacancy_info"] = '"' + vacancy_line + '"'

    surf_line = ""
    if len(data["surface_info"]) != 0:
        for i in data["surface_info"]:
            surf_line += i[0] + "," + str(round(i[1], 3)) + ";"
    info["surface_info"] = '"' + surf_line + '"'

    phonon_band_line = ""
    try:
        # Band
        frequencies = data["band_frequencies"]
        distances = data["band_distances"]
        labels = data["band_labels"]
        label_points = data["band_label_points"]

        tmp = ""
        for i in range(np.array(frequencies).shape[1]):
            tmp += ",".join(map(str, np.array(frequencies)[:, i])) + ";"

        phonon_band_line += (
            "<phonon_bandstructure_distances>'"
            + ",".join(map(str, distances))
            + "'</phonon_bandstructure_distances>"
        )

        phonon_band_line += (
            "<phonon_bandstructure_frequencies>'"
            + tmp
            + "'</phonon_bandstructure_frequencies>"
        )
        phonon_band_line += (
            "<phonon_bandstructure_labels>'"
            + ",".join(map(str, labels))
            + "'</phonon_bandstructure_labels>"
        )
        phonon_band_line += (
            "<phonon_bandstructure_label_points>'"
            + ",".join(map(str, label_points))
            + "'</phonon_bandstructure_label_points>"
        )
    except Exception as exp:
        print("Cannot obtain phonon band data.", exp)

    # Comment until 4 MB text size error
    info["phonon_band_line"] = phonon_band_line
    phonon_dos_line = ""
    try:
        freq = data["dos_freq"]
        pdos = data["dos_intensity"]
        phonon_dos_line += (
            "<phonon_dos_frequencies>'"
            + ",".join(map(str, freq))
            + "'</phonon_dos_frequencies>"
        )

        phonon_dos_line += (
            "<phonon_dos_intensity>'"
            + ",".join(map(str, pdos))
            + "'</phonon_dos_intensity>"
        )

    except Exception:
        print("Cannot obtain phonon dod data.")
        pass
    info["phonon_dos_line"] = phonon_dos_line

    return info


def write_xml(data={}, filename="temp.xml"):
    """Write XML file."""
    basic = basic_data(data)
    # name = data["jid"] + ".xml"
    f = open(filename, "w")
    basic_xml = stringdict_to_xml(basic)
    line = ""
    line = '<?xml version="1.0" encoding="UTF-8"?>\n'
    line += '<?xml-stylesheet type="text/xsl" '
    line += 'href="jarvisff.xsl"?>\n'

    line += "<basic_info>"

    line += basic_xml
    line += "</basic_info>"
    f.write(line)
    f.close()


"""
if __name__ == "__main__":
    from jarvis.io.lammps.outputs import parse_material_calculation_folder

    fold = '/rk2/knc6/JARVIS-FF/ALLOY8/
    Mishin-Ni-Al-2009.eam.alloy_nist/bulk@mp-23_fold'
    x = parse_material_calculation_folder(fold)
    write_xml(x)
    print('src',x["source_folder"].split('/')[-1].split('@')[1].split('_')[0])
"""



from jarvis.analysis.phonon.force_constants import qpoint,read_fc
import os
fc_file = os.path.join(os.path.dirname(__file__), "FORCE_CONSTANTS")

def test_fc():
   fc=read_fc(fc_file)
   assert (fc[0][0][0][0])==12.960974735
   qp=qpoint(force_constant=fc,qpt=[0, 0, 0])
   assert (qp[0][0][0][0])==12.960974735
#test_fc()   



"""Module to access or upload data in JARVIS-API."""
import requests
import pprint
import os
import glob
from jarvis.db.jsonutils import dumpjson
import string
from collections import OrderedDict
import json


class Api(object):
    """Class for handling functions for JARVIS-API."""

    def __init__(
        self,
        base_url="https://jarvis.nist.gov",
        user_info_file_path="/users/knc6/myinfo",
    ):
        """
        Intialize the class.

        Args:
            base_url: For JARVIS

            Request an account prior to access.
            Store your username and password locally
            in your a file. Format:
            ---------------------------------------
            username
            password
        """
        f = open(user_info_file_path, "r")
        lines = f.read().splitlines()
        f.close()
        self.base_url = base_url
        self.username = lines[0]
        self.password = lines[1]

    def get_data_by_hash_id(self, id=""):
        """Gat data file info for a hash_id."""
        template_upload_url = "/rest/data/" + str(id) + str("/")
        turl = self.base_url + template_upload_url + str("/?format=json")
        response = requests.get(
            turl, verify=False, auth=(self.username, self.password)
        )
        if response.ok:
            return response.json()
        else:
            raise Exception("Problem occurred while getting the file.")

    def send_keyword_query(
        self, data={"query": "JVASP-1002.xml", "title": "JVASP-1002.xml"}
    ):
        """Post general query to API."""
        template_upload_url = "/rest/data/query/keyword/"
        turl = self.base_url + template_upload_url
        response = requests.post(
            turl, verify=False, data=data, auth=(self.username, self.password)
        )
        if response.ok:
            return response.json()
        else:
            raise Exception("Problem occurred while getting the file.")
            raise Exception("Check username,password, ... details")

    def get_global_workspace_id(self):
        """Get workspace ID."""
        turl = self.base_url + "/rest/workspace/"
        response = requests.get(
            turl, verify=False, auth=(self.username, self.password)
        )
        workspace_list = json.loads(response.text)
        for workspace in workspace_list:
            if workspace["title"] == "Global Public Workspace":
                global_workspace_id = workspace["id"]
        return global_workspace_id

    def upload_xml_file(
        self,
        filename="",
        workspace_id="5df7b6defb7e53000c4652aa",
        template_id="",
    ):
        """Upload file using path of the file and schema/template id."""
        print("status: uploading data file...")
        xml_file = open(filename, "rb")
        xml_content = xml_file.read()

        xml_upload_url = "/rest/data/"
        turl = self.base_url + xml_upload_url
        base = os.path.basename(filename)
        print("Filename:" + base)
        print("Template_id:" + template_id)
        data = {
            "title": base,
            "template": template_id,
            "workspace": workspace_id,
            "xml_content": xml_content,
        }
        response = requests.post(
            turl, data=data, verify=False, auth=(self.username, self.password)
        )
        out = response.json()
        # pprint.pprint(out)
        response_code = response.status_code
        print("code:", response_code, requests.codes.ok)
        if response_code == requests.codes.created:
            print("status: uploaded.")
            upload_id = out["id"]
            print("upload_id", upload_id)
        else:
            response.raise_for_status()
            pprint.pprint(out)
            raise Exception(
                "A problem occurred when uploading the schema (Error ",
                response_code,
                ")",
            )

        return upload_id

    def delete_file_by_hash_id(self, id=""):
        """Delete files for a hash id."""
        turl = self.base_url + "/rest/data/" + str(id) + "/"
        response = requests.delete(
            turl, verify=False, auth=(self.username, self.password)
        )
        response_code = response.status_code
        print("delete response_code", response_code)
        return response_code

    def get_all_template_ids(self, title=None, only_user=False):
        """Get templates for a user or global templates/schema/xsd."""
        # title: exmaple:'testxy'
        if only_user:
            turl = self.base_url + "/rest/template-version-manager/user/"
        else:
            turl = self.base_url + "/rest/template-version-manager/global/"
        params = {}
        if title is not None:
            params["title"] = title
        response = requests.get(
            turl, params=params, auth=(self.username, self.password)
        )
        print("response", pprint.pprint(response.json()))

        response_code = response.status_code
        if response_code == requests.codes.ok:
            print("- status: successful.")
            return response.json()
        else:
            response.raise_for_status()
            raise Exception(
                "- error: a problem occurred when getting the template"
            )

    def get_all_xmls(self):
        """Get all XML data. Used for deleting."""
        return self.send_query(query=".xml")

    def delete_chunks(self, array=[]):
        """Delete a list of records."""
        # print ('array',array[0]['id'])
        for i in array:
            try:
                id = i["id"]
                self.delete_file_by_hash_id(id)
            except Exception:
                print("Failed deleting", i)
                print()
                pass

    def get_data_by_template_id(
        self, id="5f6162b4ece4b00034e4fe19", query=".xml", max_record=5
    ):
        """
        Get data by template IDs.

        For template IDs see:
        https://jarvis.nist.gov/rest/template-version-manager/global/
        and is_disabled=False
        """
        xml_upload_url = "/rest/data/query/keyword/"
        turl = self.base_url + xml_upload_url
        print("turl", turl)
        input = {"query": query}
        response = requests.post(
            turl, data=input, auth=(self.username, self.password)
        )
        out = response.json()
        params = {"page": 2}
        mem = []
        for i in out["results"]:
            if i["template"] == id:
                mem.append(i)
        if max_record is not None and len(mem) < max_record:
            while out["next"] is not None:
                response = requests.post(
                    turl,
                    data=input,
                    params=params,
                    auth=(self.username, self.password),
                )
                out = response.json()
                # mem.extend(out["results"])
                params["page"] += 1
                for i in out["results"]:
                    if i["template"] == id:
                        mem.append(i)
                        if max_record is not None:
                            if len(mem) == max_record:
                                break

        return mem

    def delete_all_records(self, query=".xml"):
        """Delete all XML files."""
        print("Deleteing all XMLs.")
        print("Hope you backed up the data.")
        xml_upload_url = "/rest/data/query/keyword/"
        turl = self.base_url + xml_upload_url
        print("turl", turl)
        input = {"query": query}
        response = requests.post(
            turl, data=input, auth=(self.username, self.password)
        )
        out = response.json()
        f = open("tmpout", "w")
        f.write(json.dumps(out))
        f.close()
        data = out["results"]
        self.delete_chunks(data)
        params = {"page": 2}

        while out["next"] is not None:
            response = requests.post(
                turl,
                data=input,
                params=params,
                auth=(self.username, self.password),
            )
            out = response.json()
            # data.extend(out["results"])
            data = out["results"]
            params["page"] += 1
            print("params", params)
            try:
                self.delete_chunks(data)
            except Exception:
                print("Failed for", data)
                pass
        return data

    def send_query(self, query="JVASP-1002.xml"):
        """Send query to API."""
        xml_upload_url = "/rest/data/query/keyword/"
        turl = self.base_url + xml_upload_url
        print("turl", turl)
        # input={"query":"JLMP-1254.xml"}
        # input={"query":"JVASP-1002.xml"}
        # input={"query":".xml"}
        input = {"query": query}
        response = requests.post(
            turl, data=input, auth=(self.username, self.password)
        )
        out = response.json()
        data = out["results"]
        params = {"page": 2}
        while out["next"] is not None:
            response = requests.post(
                turl,
                data=input,
                params=params,
                auth=(self.username, self.password),
            )
            out = response.json()
            data.extend(out["results"])
            params["page"] += 1
        return out

    def upload_blob(self, filepath=""):
        """Upload binary blob files."""
        turl = self.base_url + "/rest/blob/"

        oldid = os.path.basename(filepath)
        if all(c in string.hexdigits for c in oldid):
            newfile = oldid[25:]
        else:
            newfile = oldid

        files = {"blob": open(filepath, "rb")}

        data = {"filename": newfile}

        response = requests.post(
            turl, files=files, data=data, auth=(self.username, self.password)
        )
        if response.status_code == 201:
            print("Uploaded blob.")
            return response.json()["id"]
        else:
            response.raise_for_status()
            raise Exception("- error: a problem occurred when uploading blob")

    def get_all_blobs(self):
        """Download binary blob files."""
        turl = self.base_url + "/rest/blob/"
        response = requests.get(
            turl, verify=False, auth=(self.username, self.password)
        )
        # print ('response.json()',response.json())
        print("All blob doownload response.status_code", response.status_code)
        return response.json()

    def download_blob(self, save_file=True, bid="5f6a4bb7ece4b0002de4fb60"):
        """Download a binary blob file."""
        turl = self.base_url + "/rest/blob/" + bid + "/"
        # turl = self.base_url + "/rest/blob/download/" + bid + "/"
        response = requests.get(
            turl, verify=False, auth=(self.username, self.password)
        )
        resp = OrderedDict(response.json())
        data = requests.get(
            resp["handle"], verify=False, auth=(self.username, self.password)
        ).content
        filename = resp["filename"]
        if save_file:
            open(filename, "wb").write(data)
        print(len(data), len(filename))
        return filename, data

    def delete_blob(self, bid=""):
        """Delete a blob by id."""
        turl = self.base_url + "/rest/blob/" + str(bid) + "/"
        response = requests.delete(
            turl, verify=False, auth=(self.username, self.password)
        )
        response_code = response.status_code
        print("delete response_code", response_code)
        return response_code

    def delete_all_blobs(self):
        """Delete binary blob files."""
        data = self.get_all_blobs()
        for i in data:
            print("Deleting", i["id"])
            self.delete_blob(i["id"])

    def upload_jarvisff_xmls(
        self,
        files="/rk2/knc6/DB/XMLs/LAMMPS/*.xml",
        template_id="5f6162b4ece4b00034e4fe19",
        json_name="jarvisff_xmls.json",
    ):
        """Upload JARVIS-FF XML files."""
        mem = []
        for i in glob.glob(files):
            jid = i.split("/")[-1].split(".xml")[0]
            try:
                upload_id = self.upload_xml_file(
                    filename=i, template_id=template_id
                )
                info = {}
                info["jid"] = jid
                info["api_id"] = upload_id
                mem.append(info)

            except Exception:
                info = {}
                info["jid"] = jid
                info["api_id"] = "Failed"

                print("Failed for", i)
                pass
        if json_name is not None:
            dumpjson(filename=json_name, data=mem)
        return mem

    def upload_jarvisdft_xmls(
        self,
        files="/rk2/knc6/DB/XMLs/VASPDASK/*.xml",
        template_id="5f626925ece4b00035e5277f",
        json_name="jarvisdft_xmls.json",
    ):
        """Upload JARVIS-DFT XML files."""
        mem = []
        for i in glob.glob(files):
            jid = i.split("/")[-1].split(".xml")[0]
            print(jid)
            try:
                upload_id = self.upload_xml_file(
                    filename=i, template_id=template_id
                )
                print(jid)
                info = {}
                info["jid"] = jid
                info["api_id"] = upload_id
                mem.append(info)
            except Exception:
                info = {}
                info["jid"] = jid
                info["api_id"] = "Failed"

                print("Failed for", i)
                pass
        if json_name is not None:
            dumpjson(filename=json_name, data=mem)
        return mem


# TODO:
# workspace operations
# federated vs oai_pmh


"""
if __name__ == "__main__":

    a = Api()
    a = Api(base_url="http://test-jarvis.nist.gov")
    tid="5f624bc547d1ac011a224672"
    tid='5f6909278c6d9e011c8cc8c9'
    a.upload_xml_file(filename='JVASP-1004.xml',template_id=tid)

    a = Api()
    a = Api(base_url="https://jarvis.nist.gov")
    x = a.get_global_workspace_id()
    print("x=", x)

    a = Api()
    a = Api(base_url="https://jarvis.nist.gov")

    # a.upload_jarvisff_xmls()
    # a.upload_jarvisff_xmls()
    # a.upload_jarvisdft_xmls()
    # tid="5f626925ece4b00035e5277f"

    # a.upload_xml_file(filename='JVASP-1067.xml',template_id=tid)
    # a.upload_xml_file(filename='JVASP-664.xml',template_id=tid)
    # a.upload_xml_file(filename='JVASP-1002.xml',template_id=tid)

    # a.delete_all_records()

    # filepath="/rk2/knc6/DB/RAW_FILES/JARVIS-DFT-DFPT/JVASP-10088.zip",
    # filepath="/rk2/knc6/JARVIS-DFT/Elements-bulkk/
    # mp-149_bulk_PBEBO/MAIN-RELAX-bulk@mp-149/CHGCAR"
    # x = a.upload_blob(filepath)
    print("id", x)

    #a.upload_xml_file(filename='JVASP-1067.xml',template_id=tid)
    #a.upload_xml_file(filename='JVASP-664.xml',template_id=tid)
    #a.upload_xml_file(filename='JVASP-1002.xml',template_id=tid)

    # a.delete_all_records()

    #filepath="/rk2/knc6/DB/RAW_FILES/JARVIS-DFT-DFPT/JVASP-10088.zip",
    #filepath="/rk2/knc6/JARVIS-DFT/Elements-bulkk/
    #mp-149_bulk_PBEBO/MAIN-RELAX-bulk@mp-149/CHGCAR"
    #x = a.upload_blob(filepath)
    print ('id',x)
    a.delete_blob(x)
    # a.get_all_blobs()

    # a.upload_jarvisdft_xmls()
    # a.delete_all_blobs()
    # print(x)  # 5f61ba30ece4b00030e500c6
    # a.delete_file_by_hash_id('5f616c20ece4b00035e5135d')
    # upload_jarvisff_xmls()
    # out=a.send_query(query='JLMP-1254.xml')
    # print ('out',len(out))
    # data = a.get_data_by_template_id("5f6162b4ece4b00034e4fe19")
    # print(len(data))
    # a.upload_file(filename=x_file,template_id=id)
    # data=a.get_data_by_hash_id('5f6130e9ece4b0002fe4f8f1')
    # print (data)

    # templates=a.get_all_template_ids(title='jarvisff')
    # print ('templates',templates)
    # doc=a.get_data_by_hash_id('5f5d1578ece4b00035e4f9f5')
    # print (doc)

    import sys

    sys.exit()

    x_file = "/users/knc6/Software/Devs/jarvis/jarvis/db/testxy.xml"
    t_id = "5f395d85ece4b00031fb5cfe"
    t_id = "5f395de8ece4b00028fb5d92"
    # templates=a.get_all_template_ids()
    # templates=a.get_all_template_ids(only_user=True,title='testxy')
    id = "5f396b0dece4b00036fb5dab"
    id = "5f396ae9ece4b00035fb5d42"
    # document=a.get_data_by_hash_id(id=id)
    # print('document',document)
    # a.upload_file(filename=x_file,template_id=t_id)
    data = {"query": "JVASP-1002.xml", "title": "JVASP-1002.xml"}
    # x=a.send_keyword_query(data=data)
    id = "5f396ae9ece4b00035fb5d42"
    id = "5f396a44ece4b00035fb5d3e"
    # a.delete_file_by_hash_id(id=id)
    data = {"query": "testxy.xml", "title": "testxy.xml"}
    data = {"query": "testxy", "template": "5f395de8ece4b00028fb5d92"}
    data = {"query": "testxy"}
    result = a.send_keyword_query(data=data)
    print("result", result)
"""



from jarvis.io.vasp.outputs import Vasprun, Outcar
from jarvis.analysis.phonon.ir import ir_intensity

import os

out = Outcar(
    os.path.join(os.path.dirname(__file__), "..", "..", "io", "vasp", "OUTCAR.JVASP-39")
)
vrun = Vasprun(
    os.path.join(
        os.path.dirname(__file__), "..", "..", "io", "vasp", "vasprun.xml.JVASP-39"
    )
)


def test_ir():
    phonon_eigenvectors = vrun.dfpt_data["phonon_eigenvectors"]
    vrun_eigs = vrun.dfpt_data["phonon_eigenvalues"]
    phonon_eigenvalues = out.phonon_eigenvalues
    masses = vrun.dfpt_data["masses"]
    born_charges = vrun.dfpt_data["born_charges"]
    x, y = ir_intensity(
        phonon_eigenvectors=phonon_eigenvectors,
        phonon_eigenvalues=phonon_eigenvalues,
        masses=masses,
        born_charges=born_charges,
    )
    print (max(y))
    assert max(y) == 0.3511482090386446 
test_ir()



from jarvis.ai.pkgs.sklearn.regression import regression
from jarvis.ai.pkgs.sklearn.classification import classification
from jarvis.ai.pkgs.sklearn.hyper_params import (
    classification_regression_params,
)
from jarvis.ai.pkgs.utils import get_ml_data, binary_class_dat
from jarvis.ai.pkgs.lgbm.regression import regression as l_regression
from jarvis.ai.pkgs.lgbm.regression import parameters_dict as l_params
from jarvis.ai.pkgs.lgbm.classification import (
    classification as l_classification,
)
from jarvis.ai.descriptors.cfid import feat_names
from lightgbm import LGBMClassifier
import matplotlib.pyplot as plt
import os

plt.switch_backend("agg")

property = "exfoliation_energy"

X, Y, jid = get_ml_data(dataset="cfid_3d", ml_property=property)

X = X[0:100]
Y = Y[0:100]
jid = jid[0:100]


def test_lgbm_regression():
    params = l_params()[property]
    names = feat_names()
    info = l_regression(
        X=X,
        Y=Y,
        jid=jid,
        config=params,
        feat_names=names,
        plot=True,
        save_model=True,
    )
    assert info["reg_scores"]["mae"] < 200.0
    info = l_regression(
        X=X,
        Y=Y,
        jid=jid,
        config=params,
        preprocess=False,
        feat_names=names,
        plot=True,
        save_model=True,
    )
    cmd = 'rm my_model* *.png'
    os.system(cmd)

def test_lgbm_classification():
    property = "exfoliation_energy"
    tol = 100
    models = [LGBMClassifier(n_estimators=10, num_leaves=2)]
    info = l_classification(
        X=X,
        Y=Y,
        plot=True,
        models=models,
        preprocess=True,
        save_model=True,
        tol=tol,
    )
    cmd = 'rm my_model* *.png'
    os.system(cmd)
    assert info["LGBMClassifier"]["roc_auc"][0] > 0.0


def test_hyperparams():
    x, y = classification_regression_params()


def test_sklearn_simple_regression():

    info = regression(X, Y, plot=True)
    assert info["GradientBoostingRegressor"]["mae"] < 200.0


def test_sklearn_simple_classification():
    X_class, Y_class = binary_class_dat(X=X, Y=Y, tol=100)
    info = classification(X=X, Y=Y, tol=100, plot=True)
    assert (info["GradientBoostingClassifier"]["roc_auc"][0]) > 0.0



"""Modules for making crystallographic plane surfaces."""
from jarvis.core.atoms import Atoms
from jarvis.core.utils import ext_gcd
import numpy as np
from jarvis.analysis.structure.spacegroup import Spacegroup3D
from numpy.linalg import norm
from numpy import gcd
from collections import OrderedDict


def wulff_normals(miller_indices=[], surface_energies=[]):
    """Obtain Wulff Normals.

    Args:
         miller_indices : Miller indices

         surface_energies : corresponding surface energies

    Returns: Surface normals
    """
    max_s = min(surface_energies)
    normals = []
    for i, j in zip(miller_indices, surface_energies):
        normal = j * np.linalg.norm(i) / float(max_s)
        normals.append([normal, i])

    return normals


class Surface(object):
    """Get surface object of arbitrary atoms object and miller index."""

    def __init__(
        self,
        atoms=None,
        indices=[0, 0, 1],
        layers=3,
        vacuum=18.0,
        tol=1e-10,
        from_conventional_structure=True,
    ):
        """Initialize the class.

        Args:
             atoms: jarvis.core.Atoms object

             indices: Miller indices

             layers: Number of surface layers

             vacuum: vacuum padding

             tol: tolerance during dot product

             from_conventional_structure: whether to use the conv. atoms
        """
        self.indices = np.array(indices)
        self.from_conventional_structure = from_conventional_structure
        if self.from_conventional_structure:
            self.atoms = Spacegroup3D(atoms).conventional_standard_structure
        else:
            self.atoms = atoms
        self.tol = tol
        self.vacuum = vacuum
        self.layers = layers

    def to_dict(self):
        """Convert to a dictionary."""
        d = OrderedDict()
        d["atoms"] = self.atoms.to_dict()
        d["indices"] = self.indices
        d["tol"] = self.tol
        d["vacuum"] = self.vacuum
        d["layers"] = self.layers
        d["from_conventional_structure"] = self.from_conventional_structure
        return d

    @classmethod
    def from_dict(self, d={}):
        """Construct class from a dictionary."""
        return Surface(
            atoms=Atoms.from_dict(d["atoms"]),
            indices=d["indices"],
            tol=d["tol"],
            vacuum=d["vacuum"],
            layers=d["layers"],
            from_conventional_structure=d["from_conventional_structure"],
        )

    def make_surface(self):
        """Generate specified surface. Modified from ase package."""
        atoms = self.atoms
        h_index, k_index, l_index = self.indices
        h0, k0, l0 = self.indices == 0
        if h0 and k0 or h0 and l0 or k0 and l0:  # if two indices are zero
            if not h0:
                c1, c2, c3 = [(0, 1, 0), (0, 0, 1), (1, 0, 0)]
            if not k0:
                c1, c2, c3 = [(0, 0, 1), (1, 0, 0), (0, 1, 0)]
            if not l0:
                c1, c2, c3 = [(1, 0, 0), (0, 1, 0), (0, 0, 1)]
        else:
            p, q = ext_gcd(k_index, l_index)
            a1, a2, a3 = self.atoms.lattice_mat  # .lat_lengths()

            # constants describing the dot product of basis c1 and c2:
            # dot(c1,c2) = k1+i*k2, i in Z
            k1 = np.dot(
                p * (k_index * a1 - h_index * a2)
                + q * (l_index * a1 - h_index * a3),
                l_index * a2 - k_index * a3,
            )
            k2 = np.dot(
                l_index * (k_index * a1 - h_index * a2)
                - k_index * (l_index * a1 - h_index * a3),
                l_index * a2 - k_index * a3,
            )

            if abs(k2) > self.tol:
                i = -int(round(k1 / k2))
                p, q = p + i * l_index, q - i * k_index

            a, b = ext_gcd(p * k_index + q * l_index, h_index)

            c1 = (p * k_index + q * l_index, -p * h_index, -q * h_index)
            c2 = np.array((0, l_index, -k_index)) // abs(gcd(l_index, k_index))
            c3 = (b, a * p, a * q)
        lattice = atoms.lattice_mat  # .lat_lengths()
        basis = np.array([c1, c2, c3])
        scaled = np.linalg.solve(basis.T, np.array(atoms.frac_coords).T).T
        scaled -= np.floor(scaled + self.tol)
        new_coords = scaled
        tmp_cell = np.dot(basis, lattice)
        M = np.linalg.solve(lattice, tmp_cell)
        cart_coords = np.dot(scaled, lattice)
        new_coords = np.dot(cart_coords, M)

        new_atoms = Atoms(
            lattice_mat=tmp_cell,
            coords=new_coords,
            elements=atoms.elements,
            cartesian=True,
        )

        surf_atoms = new_atoms.make_supercell_matrix([1, 1, self.layers])
        # print("supercell_cart_coords", surf_atoms.frac_coords)

        new_lat = surf_atoms.lattice_mat  # lat_lengths()
        a1 = new_lat[0]
        a2 = new_lat[1]
        a3 = new_lat[2]
        new_lat = np.array(
            [
                a1,
                a2,
                np.cross(a1, a2)
                * np.dot(a3, np.cross(a1, a2))
                / norm(np.cross(a1, a2)) ** 2,
            ]
        )

        a1 = new_lat[0]
        a2 = new_lat[1]
        a3 = new_lat[2]
        # print("a1,a2,a3", new_lat)

        latest_lat = np.array(
            [
                (np.linalg.norm(a1), 0, 0),
                (
                    np.dot(a1, a2) / np.linalg.norm(a1),
                    np.sqrt(
                        np.linalg.norm(a2) ** 2
                        - (np.dot(a1, a2) / np.linalg.norm(a1)) ** 2
                    ),
                    0,
                ),
                (0, 0, np.linalg.norm(a3)),
            ]
        )

        M = np.linalg.solve(new_lat, latest_lat)

        new_cart_coords = surf_atoms.cart_coords  # np.dot(scaled,lattice)

        new_coords = np.dot(new_cart_coords, M)

        new_atoms = Atoms(
            lattice_mat=latest_lat,
            elements=surf_atoms.elements,
            coords=new_coords,
            cartesian=True,
        ).center_around_origin()

        frac_coords = new_atoms.frac_coords

        frac_coords[:] = frac_coords[:] % 1
        new_atoms = Atoms(
            lattice_mat=latest_lat,
            elements=surf_atoms.elements,
            coords=frac_coords,
            cartesian=False,
        )
        new_lat = new_atoms.lattice_mat
        new_cart_coords = new_atoms.cart_coords
        elements = new_atoms.elements
        new_lat[2][2] = new_lat[2][2] + self.vacuum
        with_vacuum_atoms = Atoms(
            lattice_mat=new_lat,
            elements=elements,
            coords=new_cart_coords,
            cartesian=True,
        )
        # new_atoms.center()
        # print (with_vacuum_atoms)
        return with_vacuum_atoms


"""
if __name__ == "__main__":
    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    Surface(atoms=Si, indices=[1, 1, 1]).make_surface()
    su = [
        0.8582640971273426,
        0.9334963319196496,
        0.9360461382184894,
        0.9419095687284446,
        0.9802042233627004,
        0.9875446840480956,
        1.0120634294466684,
        1.0126231880823566,
        1.0241538763302507,
        1.0315901848682645,
        1.0318271257831195,
        1.0331286888257398,
        1.0344297141291043,
        1.0388709097092674,
        1.040277640596931,
        1.042494119906149,
        1.04453679643896,
        1.0450598648770613,
        1.045076130339553,
        1.0469310544190567,
        1.0491015867538047,
        1.0495494553198788,
        1.0534717916897114,
        1.0535201391639715,
        1.054233162444997,
        1.0579157863887743,
        1.0595676718662346,
        1.0601381085497692,
        1.109580394178689,
    ]

    ml = [
        [0, 0, 1],
        [2, 0, 3],
        [2, 0, 1],
        [1, 0, 1],
        [3, 0, 2],
        [1, 0, 3],
        [3, 1, 1],
        [3, 0, 1],
        [3, 1, 3],
        [3, -1, 1],
        [3, 1, 0],
        [3, 2, 1],
        [3, 3, 1],
        [1, 0, 0],
        [2, 2, 1],
        [3, -1, 3],
        [3, -1, 2],
        [3, 3, 2],
        [3, 2, 2],
        [2, -1, 3],
        [3, 2, 0],
        [3, 2, 3],
        [1, 1, 1],
        [1, 0, 2],
        [3, 1, 2],
        [2, -1, 2],
        [3, -1, 0],
        [2, 2, 3],
        [1, 1, 0],
    ]
    nm = wulff_normals(miller_indices=ml, surface_energies=su)
    print(nm)
    from jarvis.core.lattice import Lattice
    lat = Lattice([[4.05, 0, 0], [0, 4.05, 0], [0, 0, 4.05]])
    pmg_wulff = WulffShape(lat, ml, su)
    print(pmg_wulff.facets)
"""



"""Modules for making point-defect vacancies."""
import pprint
from collections import OrderedDict
from jarvis.analysis.structure.spacegroup import Spacegroup3D
from jarvis.core.utils import rand_select
from jarvis.core.atoms import Atoms


class Vacancy(object):
    """Obtain vacancy defects in Atoms class using Wyckoff data."""

    def __init__(
        self,
        atoms=None,
        defect_structure=None,
        defect_index=None,
        wyckoff_multiplicity=None,
        symbol=None,
    ):
        """
        Initialize the method.

        Arguments are given below.
        Args:
            atoms: jarvis.core.Atoms object.

            defect_index: atoms index for defect.

            defect_structure:  Atoms with defect.

            wyckoff_multiplicity: Wyckoff multiplicity.

            symbol: Elemenyt symbol.
        """
        self._atoms = atoms
        self._defect_index = defect_index
        self._defect_structure = defect_structure
        self._wyckoff_multiplicity = wyckoff_multiplicity
        self._symbol = symbol

    @classmethod
    def from_dict(self, d={}):
        """Load from a dictionary."""
        return Vacancy(
            atoms=Atoms.from_dict(d["atoms"]),
            defect_structure=Atoms.from_dict(d["defect_structure"]),
            defect_index=d["defect_index"],
            wyckoff_multiplicity=d["wyckoff_multiplicity"],
            symbol=d["symbol"],
        )

    def generate_defects(
        self, enforce_c_size=10.0, on_conventional_cell=False, extend=1
    ):
        """Provide function to generate defects."""
        atoms = self._atoms
        if on_conventional_cell:
            atoms = Spacegroup3D(atoms).conventional_standard_structure
        a = atoms.lattice.lat_lengths()[0]
        b = atoms.lattice.lat_lengths()[1]
        c = atoms.lattice.lat_lengths()[2]
        if enforce_c_size is not None:
            dim1 = int(float(enforce_c_size) / float(a)) + extend
            dim2 = int(float(enforce_c_size) / float(b)) + extend
            dim3 = int(float(enforce_c_size) / float(c)) + extend
            # atoms = atoms.make_supercell([dim1, dim2, dim3])
            supercell_size = [dim1, dim2, dim3]

        spg = Spacegroup3D(atoms)
        wyckoffs = spg._dataset["wyckoffs"]
        atoms.props = wyckoffs
        supercell = atoms.make_supercell(supercell_size)
        props = rand_select(supercell.props)
        vacs = []
        for i, j in props.items():
            defect_strt = supercell.remove_site_by_index(j)
            vac = Vacancy(
                atoms=supercell,
                defect_structure=defect_strt,
                defect_index=j,
                wyckoff_multiplicity=i,
                symbol=supercell.elements[j],
            )
            vacs.append(vac)
        return vacs

    def to_dict(self):
        """Convert to a dictionary."""
        d = OrderedDict()
        d["atoms"] = self._atoms.to_dict()
        if self._defect_structure is not None:
            d["defect_structure"] = self._defect_structure.to_dict()
        else:
            d["defect_structure"] = None
        d["defect_index"] = self._defect_index
        d["wyckoff_multiplicity"] = self._wyckoff_multiplicity
        d["symbol"] = self._symbol
        return d

    def __repr__(self, indent=2):
        """Representation of the class as dict."""
        return pprint.pformat(self.to_dict(), indent=indent)


"""
if __name__ == "__main__":
    from jarvis.io.vasp.inputs import Poscar

    box = [[2.715, 2.715, 0], [0, 2.715, 2.715], [2.715, 0, 2.715]]
    coords = [[0, 0, 0], [0.25, 0.25, 0.25]]
    elements = ["Si", "Si"]
    Si = Atoms(lattice_mat=box, coords=coords, elements=elements)
    Si = Poscar.from_file(
        "/rk2/knc6/JARVIS-DFT/2D-bulk/mp-1143_bulk_LDA/MAIN-ELASTIC-bulk@mp_1143/POSCAR"
    ).atoms
    vacs = Vacancy(atoms=Si).generate_defects()
    for i in vacs:
        print(i)
    spg = Spacegroup3D(Si)
    cvn = spg.conventional_standard_structure
    spg = Spacegroup3D(cvn)
    props = spg._dataset["wyckoffs"]
    Si.props = props
    ss = Si.make_supercell([2, 2, 2])
    props = ss.props
    # print (rand_select(props))
"""



"""Set of useful utility functions."""

from collections import OrderedDict
from collections import defaultdict
import random
import numpy as np
import math
import xmltodict
import re


def xml_to_dict(fname):
    """Parse XML file."""
    with open(fname, "r") as f:
        data = xmltodict.parse(f.read())
    return data


def get_counts(array=["W", "W", "Mo", "Mo", "S", "S"]):
    """
    Get number of unique elements and their counts.

    Uses OrderedDict.

    Args:
         array of elements

    Returns:
         ordereddict, e.g.OrderedDict([('W', 2), ('Mo', 2), ('S', 2)])
    """
    uniqe_els = []
    for i in array:
        if i not in uniqe_els:
            uniqe_els.append(i)
    info = OrderedDict()
    for i in uniqe_els:
        info.setdefault(i, 0)
    for i in array:
        info[i] += 1
    return info


def gcd(a, b):
    """Calculate the Greatest Common Divisor of a and b.

    Unless b==0, the result will have the same sign as b (so that when
    b is divided by it, the result comes out positive).
    """
    while b:
        a, b = b, a % b
    return a


def ext_gcd(a, b):
    """GCD module from ase."""
    if b == 0:
        return 1, 0
    elif a % b == 0:
        return 0, 1
    else:
        x, y = ext_gcd(b, a % b)
        return y, x - y * (a // b)


def rand_select(x=[]):
    """Select randomly with index info."""
    info = {}
    for i, ii in enumerate(x):
        info.setdefault(ii, []).append(i)
    selected = {}
    for i, j in info.items():
        chosen = random.choice(j)
        selected.setdefault(i, chosen)
    return selected


def rec_dict():
    """Make a recursion dictionary."""
    return defaultdict(rec_dict)


def random_colors(number_of_colors=110):
    """Generate random colors for atom coloring."""
    colors = [
        "#" + "".join([random.choice("0123456789ABCDEF") for j in range(6)])
        for i in range(number_of_colors)
    ]
    color_dict = {}
    for i, ii in enumerate(colors):
        color_dict[i] = ii
    return color_dict


def get_angle(
    a=np.array([1, 2, 3]), b=np.array([4, 5, 6]), c=np.array([7, 8, 9])
):
    """Get angle between three vectors."""
    # theta = argcos(x.y/(|x||y|))
    cos = np.dot((a - b), (c - b)) / (
        np.linalg.norm((a - b)) * np.linalg.norm((c - b))
    )
    if cos <= -1.0:
        cos = cos + 0.000001
    if cos >= 1.0:
        cos = cos - 0.000001
    angle = math.degrees(math.acos(cos))
    return angle


def recast_array_on_uniq_array_elements(
    uniq=["Si", "Al", "O"],
    arr=["Si", "Si", "Al", "Al", "Si", "O", "O", "O", "O"],
):
    """Recast array on uniq array elements."""
    info = {}
    for i, ii in enumerate(uniq):
        for j, jj in enumerate(arr):
            if ii == jj:
                info.setdefault(ii, []).append(j)
    return info


def lorentzian(x, y, x0, gamma):
    """Get Lorentzian of a function."""
    return (y / math.pi) * (
        (0.5 * gamma) / ((x - x0) ** 2 + (0.5 * gamma) ** 2)
    )


# color_dict=random_colors()
def stringdict_to_xml(d={}, enforce_string=False):
    """Convert string dictionary to XML."""
    line = ""
    for i, j in d.items():
        if enforce_string:
            line += "<" + str(i) + ">'" + str(j) + "'</" + str(i) + ">"
        else:
            line += "<" + str(i) + ">" + str(j) + "</" + str(i) + ">"
    return line


def array_to_string(arr=[]):
    """Convert 1D arry to string."""
    return ",".join(map(str, arr))


def chunks(lst, n):
    """Split successive n-sized chunks from list."""
    x = []
    for i in range(0, len(lst), n):
        x.append(lst[i : i + n])
    return x


def check_match(a, b, tol=1e-8):
    """Check if a and b are the same, taking into account PBCs."""
    if abs(a[0] - b[0]) < tol or abs(abs(a[0] - b[0]) - 1) < tol:
        if abs(a[1] - b[1]) < tol or abs(abs(a[1] - b[1]) - 1) < tol:
            if abs(a[2] - b[2]) < tol or abs(abs(a[2] - b[2]) - 1) < tol:
                return True
    return False


def update_dict(main={}, extra={}):
    """Return update dictionary."""
    # Helper function for dict.update method
    tmp = main.copy()
    for i, j in extra.items():
        tmp[i] = j
    return tmp


def get_new_coord_for_xyz_sym(frac_coord=[], xyz_string=""):
    """Obtain new coord from xyz string."""
    affine_matrix = parse_xyz_string(xyz_string)
    coord = operate_affine(frac_coord, affine_matrix)
    coord = np.array([i - math.floor(i) for i in coord])
    return coord


def check_duplicate_coords(coords=[], coord=[]):
    """Check if a coordinate exists."""
    positive = False
    for i in coords:
        tmp = check_match(i, coord)
        if tmp:
            positive = True
    return positive


def parse_xyz_string(xyz_string):
    """
    Convert xyz info to translation and rotation vectors.

    Adapted from pymatgen.
    Args:
        xyz_string: string of the form 'x, y, z', '-x, -y, z',
            '-2y+1/2, 3x+1/2, z-y+1/2', etc.
    Returns:
        translation and rotation vectors.
    """
    rot_matrix = np.zeros((3, 3))
    trans = np.zeros(3)
    toks = xyz_string.strip().replace(" ", "").lower().split(",")
    re_rot = re.compile(r"([+-]?)([\d\.]*)/?([\d\.]*)([x-z])")
    re_trans = re.compile(r"([+-]?)([\d\.]+)/?([\d\.]*)(?![x-z])")
    for i, tok in enumerate(toks):
        # build the rotation matrix
        for m in re_rot.finditer(tok):
            factor = -1 if m.group(1) == "-" else 1
            if m.group(2) != "":
                factor *= (
                    float(m.group(2)) / float(m.group(3))
                    if m.group(3) != ""
                    else float(m.group(2))
                )
            j = ord(m.group(4)) - 120
            rot_matrix[i, j] = factor
        # build the translation vector
        for m in re_trans.finditer(tok):
            factor = -1 if m.group(1) == "-" else 1
            num = (
                float(m.group(2)) / float(m.group(3))
                if m.group(3) != ""
                else float(m.group(2))
            )
            trans[i] = num * factor
    affine_matrix = np.eye(4)
    affine_matrix[0:3][:, 0:3] = rot_matrix
    affine_matrix[0:3][:, 3] = trans
    return affine_matrix


def operate_affine(cart_coord=[], affine_matrix=[]):
    """Operate affine method."""
    affine_point = np.array([cart_coord[0], cart_coord[1], cart_coord[2], 1])
    return np.dot(np.array(affine_matrix), affine_point)[0:3]


# def is_xml_valid(xsd="jarvisdft.xsd", xml="JVASP-1002.xml"):
#    """Check if XML is valid."""
#    xml_file = etree.parse(xml)
#    xml_validator = etree.XMLSchema(file=xsd)
#    is_valid = xml_validator.validate(xml_file)
#    return is_valid



"""
Simple ML models for classifcation and regression.

Designed for educational purposes only
"""
from collections import defaultdict
from sklearn.metrics import roc_curve, auc
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import label_binarize
from sklearn.preprocessing import StandardScaler
from sklearn.feature_selection import VarianceThreshold
from sklearn.pipeline import Pipeline
from sklearn.neural_network import MLPClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.gaussian_process import GaussianProcessClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.ensemble import (
    RandomForestClassifier,
    AdaBoostClassifier,
    GradientBoostingClassifier,
)
from sklearn.svm import SVC
import pickle
import joblib
import matplotlib.pyplot as plt
from jarvis.ai.pkgs.utils import binary_class_dat

simple_class_models = [
    DecisionTreeClassifier(),
    MLPClassifier(),
    GradientBoostingClassifier(),
    KNeighborsClassifier(),
    GaussianProcessClassifier(),
    RandomForestClassifier(),
    AdaBoostClassifier(),
    SVC(),
]


def classify_roc_ml(
    X=[],
    y=[],
    classes=[0, 1, 2],
    names=["High val", "Low val", ""],
    n_plot=1,
    method="",
    preprocess=True,
    plot=False,
    test_size=0.1,
):
    """
    Classifcation module for ROC curve for upto three classes.

    It can be expanded in more classes as well.
    Args:
        X: input feature vectors

        y: target data obtained from binary_class_dat

        classes: dummy classes

        names: name holders for the target data

        method: ML method

        preprocess: whether to apply standard preprocessing techniques

        plot: whether to plot the ROC curve
    """
    if plot:
        plt.close()
        plt.rcParams.update({"font.size": 22})
        plt.figure(figsize=(12, 8))

    y = label_binarize(y, classes=classes)
    n_classes = y.shape[1]

    pipe = Pipeline(
        [
            ("stdscal", StandardScaler()),
            ("vart", VarianceThreshold(1e-4)),
            ("est", method),
        ]
    )
    if preprocess:
        model = pipe
    else:
        model = method
    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=test_size, random_state=0
    )
    classifier = OneVsRestClassifier(model)
    if hasattr(model, "decision_function"):
        y_score = classifier.fit(X_train, y_train).decision_function(X_test)
    else:
        y_score = classifier.fit(X_train, y_train).predict_proba(X_test)

    lw = 3

    fpr = dict()
    tpr = dict()
    roc_auc = dict()
    for i in range(n_classes):
        fpr[i], tpr[i], _ = roc_curve(y_test[:, i], y_score[:, i])
        roc_auc[i] = auc(fpr[i], tpr[i])
    colors = ["blue", "red", "green"]
    count = 0
    for i, color, name in zip(range(n_classes), colors, names):

        if name != "":
            if count < n_plot:
                count = count + 1
                if plot:
                    plt.plot(
                        fpr[i],
                        tpr[i],
                        color=color,
                        lw=lw,
                        label="ROC  {0} (area = {1:0.2f})"
                        "".format(name, roc_auc[i]),
                    )
    if plot:
        plt.plot([0, 1], [0, 1], "k--", lw=lw)
        plt.xlim([-0.05, 1.0])
        plt.ylim([0.0, 1.05])
        plt.xlabel("False Positive Rate")
        plt.ylabel("True Positive Rate")
        plt.legend(loc="lower right")
        plt.show()
    model = classifier.fit(X, y)
    return model, roc_auc


def classification(
    X=[],
    Y=[],
    tol=100,
    plot=False,
    preprocess=True,
    models=simple_class_models,
    model_name="my_model",
    save_model=False,
):
    """Quickly train some of the classifcation algorithms in scikit-learn."""
    X_class, Y_class = binary_class_dat(X=X, Y=Y, tol=tol)
    info = defaultdict()
    for i in models:
        m, r = classify_roc_ml(
            X=X_class, y=Y_class, method=i, preprocess=preprocess, plot=plot
        )
        print(type(i).__name__, r[0])
        info[type(i).__name__] = {}
        info[type(i).__name__]["roc_auc"] = r
        if save_model:
            pk = (
                str(model_name)
                + "_"
                + str(type(i).__name__)
                + "_"
                + str(".pk")
            )
            jb = (
                str(model_name)
                + "_"
                + str(type(i).__name__)
                + "_"
                + str(".jb")
            )
            pickle.dump(m, open(pk, "wb"))
            joblib.dump(m, jb)
    return info


"""
if __name__ == "__main__":
    from jarvis.ai.pkgs.utils import get_ml_data
    X, Y, jid  = get_ml_data(dataset = 'cfid_3d',
                             ml_property='exfoliation_energy')
    X_class, Y_class = binary_class_dat(X=X, Y=Y, tol=100)
    info = classification(X=X,Y=Y,tol=100, save_model=True)
    #print (info)
    #print ()
    #print (info['GradientBoostingClassifier']['roc_auc'][0])
"""



"""Module to generate networkx graphs."""
from jarvis.core.atoms import get_supercell_dims
from jarvis.core.specie import Specie
from jarvis.core.utils import random_colors
import numpy as np
from collections import OrderedDict
from jarvis.analysis.structure.neighbors import NeighborsAnalysis
from jarvis.core.specie import get_node_attributes
import warnings
from typing import List, Tuple
from jarvis.core.atoms import Atoms
from collections import defaultdict

try:
    import torch
    from tqdm import tqdm
    import dgl
except Exception as exp:
    warnings.warn("dgl/torch/tqdm is not installed.", exp)
    pass


class Graph(object):
    """Generate a graph object."""

    def __init__(
        self,
        nodes=[],
        node_attributes=[],
        edges=[],
        edge_attributes=[],
        color_map=None,
        labels=None,
    ):
        """
        Initialize the graph object.

        Args:
            nodes: IDs of the graph nodes as integer array.

            node_attributes: node features as multi-dimensional array.

            edges: connectivity as a (u,v) pair where u is
                   the source index and v the destination ID.

            edge_attributes: attributes for each connectivity.
                             as simple as euclidean distances.
        """
        self.nodes = nodes
        self.node_attributes = node_attributes
        self.edges = edges
        self.edge_attributes = edge_attributes
        self.color_map = color_map
        self.labels = labels

    @staticmethod
    def atom_dgl_multigraph(
        atoms=None,
        cutoff=8.0,
        max_neighbors=12,
        atom_features="cgcnn",
        enforce_undirected=False,
        max_attempts=3,
        id=None,
    ):
        """Obtain a DGLGraph for Atoms object."""
        all_neighbors = atoms.get_all_neighbors(r=cutoff)
        # if a site has too few neighbors, increase the cutoff radius
        min_nbrs = min(len(neighborlist) for neighborlist in all_neighbors)
        # print('min_nbrs,max_neighbors=',min_nbrs,max_neighbors)
        attempt = 0
        while min_nbrs < max_neighbors:
            print("extending cutoff radius!", attempt, cutoff, id)
            lat = atoms.lattice
            r_cut = max(cutoff, lat.a, lat.b, lat.c)
            attempt += 1
            if attempt >= max_attempts:
                atoms = atoms.make_supercell([2, 2, 2])
                print(
                    "Making supercell, exceeded,attempts",
                    max_attempts,
                    "cutoff",
                    r_cut,
                    id,
                )
            cutoff = r_cut
            all_neighbors = atoms.get_all_neighbors(r=cutoff)
            min_nbrs = min(len(neighborlist) for neighborlist in all_neighbors)
            # return Graph.atom_dgl_multigraph(
            #    atoms, r_cut, max_neighbors, atom_features
            # )

        # build up edge list
        # Currently there's no guarantee that this creates undirected graphs
        # An undirected solution would build the full edge list where nodes are
        # keyed by (index,image), and ensure each edge has a complementary edge

        # indeed,JVASP-59628 is an example of a calculation where this produces
        # a graph where one site has no incident edges!

        # build an edge dictionary u -> v
        # so later we can run through the dictionary
        # and remove all pairs of edges
        # so what's left is the odd ones out
        edges = defaultdict(list)

        u, v, r = [], [], []
        for site_idx, neighborlist in enumerate(all_neighbors):

            # sort on distance
            neighborlist = sorted(neighborlist, key=lambda x: x[2])

            ids = np.array([nbr[1] for nbr in neighborlist])
            distances = np.array([nbr[2] for nbr in neighborlist])
            c = np.array([nbr[3] for nbr in neighborlist])

            # find the distance to the k-th nearest neighbor
            max_dist = distances[max_neighbors - 1]

            # keep all edges out to the neighbor shell of the k-th neighbor
            ids = ids[distances <= max_dist]
            c = c[distances <= max_dist]
            distances = distances[distances <= max_dist]

            u.append([site_idx] * len(ids))
            v.append(ids)
            r.append(distances)

            # keep track of cell-resolved edges
            # to enforce undirected graph construction
            for dst, cell_id in zip(ids, c):
                u_key = f"{site_idx}-(0.0, 0.0, 0.0)"
                v_key = f"{dst}-{tuple(cell_id)}"
                edge_key = tuple(sorted((u_key, v_key)))
                edges[edge_key].append((site_idx, dst))

        if enforce_undirected:
            # add complementary edges to unpaired edges
            for edge_pair in edges.values():
                if len(edge_pair) == 1:
                    src, dst = edge_pair[0]
                    u.append(dst)  # swap the order!
                    v.append(src)
                    r.append(atoms.raw_distance_matrix[src, dst])

        u = torch.tensor(np.hstack(u))
        v = torch.tensor(np.hstack(v))
        r = torch.tensor(np.hstack(r)).type(torch.get_default_dtype())

        # build up atom attribute tensor
        species = atoms.elements
        node_features = torch.tensor(
            [
                get_node_attributes(s, atom_features=atom_features)
                for s in species
            ]
        ).type(torch.get_default_dtype())

        g = dgl.graph((u, v))
        g.ndata["atom_features"] = node_features
        g.edata["bondlength"] = r

        return g

    @staticmethod
    def from_atoms(
        atoms=None,
        get_prim=False,
        zero_diag=False,
        node_atomwise_angle_dist=False,
        node_atomwise_rdf=False,
        features="basic",
        enforce_c_size=10.0,
        max_n=100,
        max_cut=5.0,
        verbose=False,
        make_colormap=True,
    ):
        """
        Get Networkx graph. Requires Networkx installation.

        Args:
             atoms: jarvis.core.Atoms object.

             rcut: cut-off after which distance will be set to zero
                   in the adjacency matrix.

             features: Node features.
                       'atomic_number': graph with atomic numbers only.
                       'cfid': 438 chemical descriptors from CFID.
                       'basic':10 features
                       'atomic_fraction': graph with atomic fractions
                                         in 103 elements.
                       array: array with CFID chemical descriptor names.
                       See: jarvis/core/specie.py

             enforce_c_size: minimum size of the simulation cell in Angst.
        """
        if get_prim:
            atoms = atoms.get_primitive_atoms
        dim = get_supercell_dims(atoms=atoms, enforce_c_size=enforce_c_size)
        atoms = atoms.make_supercell(dim)

        adj = np.array(atoms.raw_distance_matrix.copy())

        # zero out edges with bond length greater than threshold
        adj[adj >= max_cut] = 0

        if zero_diag:
            np.fill_diagonal(adj, 0.0)
        nodes = np.arange(atoms.num_atoms)
        if features == "atomic_number":
            node_attributes = np.array(
                [[np.array(Specie(i).Z)] for i in atoms.elements],
                dtype="float",
            )
        if features == "atomic_fraction":
            node_attributes = []
            fracs = atoms.composition.atomic_fraction_array
            for i in fracs:
                node_attributes.append(np.array([float(i)]))
            node_attributes = np.array(node_attributes)

        elif features == "basic":
            feats = [
                "Z",
                "coulmn",
                "row",
                "X",
                "atom_rad",
                "nsvalence",
                "npvalence",
                "ndvalence",
                "nfvalence",
                "first_ion_en",
                "elec_aff",
            ]
            node_attributes = []
            for i in atoms.elements:
                tmp = []
                for j in feats:
                    tmp.append(Specie(i).element_property(j))
                node_attributes.append(tmp)
            node_attributes = np.array(node_attributes, dtype="float")
        elif features == "cfid":
            node_attributes = np.array(
                [np.array(Specie(i).get_descrp_arr) for i in atoms.elements],
                dtype="float",
            )
        elif isinstance(features, list):
            node_attributes = []
            for i in atoms.elements:
                tmp = []
                for j in features:
                    tmp.append(Specie(i).element_property(j))
                node_attributes.append(tmp)
            node_attributes = np.array(node_attributes, dtype="float")
        else:
            print("Please check the input options.")
        if node_atomwise_rdf or node_atomwise_angle_dist:
            nbr = NeighborsAnalysis(
                atoms, max_n=max_n, verbose=verbose, max_cut=max_cut
            )
        if node_atomwise_rdf:
            node_attributes = np.concatenate(
                (node_attributes, nbr.atomwise_radial_dist()), axis=1
            )
            node_attributes = np.array(node_attributes, dtype="float")
        if node_atomwise_angle_dist:
            node_attributes = np.concatenate(
                (node_attributes, nbr.atomwise_angle_dist()), axis=1
            )
            node_attributes = np.array(node_attributes, dtype="float")

        # construct edge list
        uv = []
        edge_features = []
        for ii, i in enumerate(atoms.elements):
            for jj, j in enumerate(atoms.elements):
                bondlength = adj[ii, jj]
                if bondlength > 0:
                    uv.append((ii, jj))
                    edge_features.append(bondlength)

        edge_attributes = edge_features

        if make_colormap:
            sps = atoms.uniq_species
            color_dict = random_colors(number_of_colors=len(sps))
            new_colors = {}
            for i, j in color_dict.items():
                new_colors[sps[i]] = j
            color_map = []
            for ii, i in enumerate(atoms.elements):
                color_map.append(new_colors[i])
        return Graph(
            nodes=nodes,
            edges=uv,
            node_attributes=np.array(node_attributes),
            edge_attributes=np.array(edge_attributes),
            color_map=color_map,
        )

    def to_networkx(self):
        """Get networkx representation."""
        import networkx as nx

        graph = nx.DiGraph()
        graph.add_nodes_from(self.nodes)
        graph.add_edges_from(self.edges)
        for i, j in zip(self.edges, self.edge_attributes):
            graph.add_edge(i[0], i[1], weight=j)
        return graph

    @property
    def num_nodes(self):
        """Return number of nodes in the graph."""
        return len(self.nodes)

    @property
    def num_edges(self):
        """Return number of edges in the graph."""
        return len(self.edges)

    @classmethod
    def from_dict(self, d={}):
        """Constuct class from a dictionary."""
        return Graph(
            nodes=d["nodes"],
            edges=d["edges"],
            node_attributes=d["node_attributes"],
            edge_attributes=d["edge_attributes"],
            color_map=d["color_map"],
            labels=d["labels"],
        )

    def to_dict(self):
        """Provide dictionary representation of the Graph object."""
        info = OrderedDict()
        info["nodes"] = np.array(self.nodes).tolist()
        info["edges"] = np.array(self.edges).tolist()
        info["node_attributes"] = np.array(self.node_attributes).tolist()
        info["edge_attributes"] = np.array(self.edge_attributes).tolist()
        info["color_map"] = np.array(self.color_map).tolist()
        info["labels"] = np.array(self.labels).tolist()
        return info

    def __repr__(self):
        """Provide representation during print statements."""
        return "Graph({})".format(self.to_dict())

    @property
    def adjacency_matrix(self):
        """Provide adjacency_matrix of graph."""
        A = np.zeros((self.num_nodes, self.num_nodes))
        for edge, a in zip(self.edges, self.edge_attributes):
            A[edge] = a
        return A


class Standardize(torch.nn.Module):
    """Standardize atom_features: subtract mean and divide by std."""

    def __init__(self, mean: torch.Tensor, std: torch.Tensor):
        """Register featurewise mean and standard deviation."""
        super().__init__()
        self.mean = mean
        self.std = std

    def forward(self, g: dgl.DGLGraph):
        """Apply standardization to atom_features."""
        g = g.local_var()
        h = g.ndata.pop("atom_features")
        g.ndata["atom_features"] = (h - self.mean) / self.std
        return g


class StructureDataset(torch.utils.data.Dataset):
    """Dataset of crystal DGLGraphs."""

    def __init__(
        self,
        structures,
        targets,
        ids=None,
        cutoff=8.0,
        maxrows=np.inf,
        atom_features="atomic_number",
        transform=None,
        enforce_undirected=False,
        max_neighbors=12,
    ):
        """Initialize the class."""
        self.graphs = []
        self.labels = []
        self.ids = []

        for idx, (structure, target, id) in enumerate(
            tqdm(zip(structures, targets, ids))
        ):

            if idx >= maxrows:
                break
            a = Atoms.from_dict(structure)
            g = Graph.atom_dgl_multigraph(
                a,
                atom_features=atom_features,
                enforce_undirected=enforce_undirected,
                cutoff=cutoff,
                max_neighbors=max_neighbors,
                id=id,
            )

            self.graphs.append(g)
            self.labels.append(target)
            self.ids.append(id)

        self.labels = torch.tensor(self.labels).type(torch.get_default_dtype())
        self.transform = transform

    def __len__(self):
        """Get length."""
        return self.labels.shape[0]

    def __getitem__(self, idx):
        """Get StructureDataset sample."""
        g = self.graphs[idx]
        label = self.labels[idx]

        if self.transform:
            g = self.transform(g)

        return g, label

    def setup_standardizer(self):
        """Atom-wise feature standardization transform."""
        x = torch.cat([g.ndata["atom_features"] for g in self.graphs])
        self.atom_feature_mean = x.mean(0)
        self.atom_feature_std = x.std(0)

        self.transform = Standardize(
            self.atom_feature_mean, self.atom_feature_std
        )

    @staticmethod
    def collate(samples: List[Tuple[dgl.DGLGraph, torch.Tensor]]):
        """Dataloader helper to batch graphs cross `samples`."""
        graphs, labels = map(list, zip(*samples))
        batched_graph = dgl.batch(graphs)
        return batched_graph, torch.tensor(labels)


"""
if __name__ == "__main__":
    from jarvis.core.atoms import Atoms
    from jarvis.db.figshare import get_jid_data

    atoms = Atoms.from_dict(get_jid_data("JVASP-664")["atoms"])
    g = Graph.from_atoms(
        atoms=atoms,
        features="basic",
        get_prim=True,
        zero_diag=True,
        node_atomwise_angle_dist=True,
        node_atomwise_rdf=True,
    )
    g = Graph.from_atoms(
        atoms=atoms,
        features="cfid",
        get_prim=True,
        zero_diag=True,
        node_atomwise_angle_dist=True,
        node_atomwise_rdf=True,
    )
    g = Graph.from_atoms(
        atoms=atoms,
        features="atomic_number",
        get_prim=True,
        zero_diag=True,
        node_atomwise_angle_dist=True,
        node_atomwise_rdf=True,
    )
    g = Graph.from_atoms(atoms=atoms, features="basic")
    g = Graph.from_atoms(
        atoms=atoms, features=["Z", "atom_mass", "max_oxid_s"]
    )
    g = Graph.from_atoms(atoms=atoms, features="cfid")
    # print(g)
    d = g.to_dict()
    g = Graph.from_dict(d)
    num_nodes = g.num_nodes
    num_edges = g.num_edges
    print(num_nodes, num_edges)
    assert num_nodes == 48
    assert num_edges == 2304
    assert len(g.adjacency_matrix) == 2304
    # graph, color_map = get_networkx_graph(atoms)
    # nx.draw(graph, node_color=color_map, with_labels=True)
    # from jarvis.analysis.structure.neighbors import NeighborsAnalysis
"""



"""Module for LightGBM classification."""

from jarvis.ai.pkgs.sklearn.classification import classification as sk_cl


def classification(
    X=[],
    Y=[],
    tol=100,
    plot=False,
    preprocess=False,
    models=[],
    model_name="my_model",
    save_model=False,
):
    """Provide function for classification models."""
    info = sk_cl(
        X=X,
        Y=Y,
        tol=tol,
        preprocess=preprocess,
        models=models,
        model_name=model_name,
        save_model=save_model,
    )
    return info


"""
if __name__ == "__main__":
    from jarvis.ai.pkgs.utils import get_ml_data, binary_class_dat
    property = "exfoliation_energy"
    tol=100
    #property = 'optb88vdw_bandgap'
    #tol=0.05
    X, Y, jid = get_ml_data(dataset="cfid_3d", ml_property=property)
    #X_class, Y_class = binary_class_dat(X=X, Y=Y, tol=tol)
    print ('lennnn',len(X))
    #models = [LGBMClassifier(n_estimators=1000,max_depth=50,num_leaves=100)]
    models = [LGBMClassifier()]
    info = classification(X=X, Y=Y, models=models,
    preprocess=False, save_model=False, tol=tol)
    print (info['LGBMClassifier']['roc_auc'][0])
"""



"""Module to simulate STM with Tershoff-Hamann approach."""
# Reference: https://www.nature.com/articles/s41597-021-00824-y
import matplotlib.pyplot as plt
from jarvis.io.vasp.outputs import Chgcar
import numpy as np
import matplotlib.transforms as mtransforms


class TersoffHamannSTM(object):
    """Generate constant height and constant current STM images."""

    def __init__(self, chg_name="PARCHG", min_size=50.0, skew=True, zcut=None):
        """Initialize class with pathe of PARCHG and other input params."""
        chgcar = Chgcar(filename=chg_name)
        self.atoms = chgcar.atoms
        self.dim = chgcar.dim
        self.zmaxp = self.atoms.cart_coords[:, 2].max()
        self.nz = self.dim[2]
        volume = self.atoms.volume
        tmp = chgcar.chg[-1] * volume
        chg = tmp.reshape(self.dim[::-1]).T
        self.chg = chg
        self.a = self.atoms.lattice.a
        self.b = self.atoms.lattice.b
        self.c = self.atoms.lattice.c
        self.skew = skew
        self.zcut = zcut
        z_frac_coords = self.atoms.frac_coords[:, 2]
        z_frac_coords_moved = []
        for i in z_frac_coords:
            if i > 0.5:
                i = i - 1
            elif i < -0.5:
                i = i + 1
            z_frac_coords_moved.append(i)
        self.zmaxp = max(np.array(z_frac_coords_moved) * self.c)
        rep_x = int(min_size / self.a) + 1
        rep_y = int(min_size / self.b) + 1
        self.repeat = [rep_x, rep_y]
        self.scell = self.atoms.make_supercell_matrix([rep_x, rep_y, 1])

    def constant_height(self, tol=2, filename="testh.png"):
        """Get iso-height image."""
        if not self.zcut:
            self.zcut = int((self.zmaxp + tol) / self.c * self.nz)
        img_ext = np.tile(self.chg[:, :, self.zcut], self.repeat)
        exts = (0, self.a * self.repeat[0], 0, self.b * self.repeat[1])
        plt.close()
        fig, ax = plt.subplots()
        plt.xticks([])
        plt.yticks([])
        if self.skew:
            tmp = 90 - self.atoms.lattice.angles[2]
        else:
            tmp = 0
        data = self.get_plot(
            ax, img_ext, exts, mtransforms.Affine2D().skew_deg(tmp, 0),
        )
        info = {}
        info["img_ext"] = img_ext
        info["data"] = data
        info["scell"] = self.scell
        info["zcut"] = self.zcut
        fig.subplots_adjust(bottom=0, top=1, left=0.0, right=1)
        plt.savefig(filename, bbox_inches="tight", pad_inches=0.0, dpi=240)
        plt.close()

        return info

    def constant_current(self, tol=2, pc=None, ext=0.15, filename="testc.png"):
        """Return the constant-current cut the charge density."""
        zmax_ind = int(self.zmaxp / self.c * self.nz) + 1
        # Find what z value is near the current, and take avergae
        if not self.zcut:
            self.zcut = int((self.zmaxp + tol) / self.c * self.nz)
        zext = int(self.nz * ext)
        zcut_min = self.zcut - zext
        if zcut_min < zmax_ind:
            zcut_min = zmax_ind
        zcut_max = self.zcut + zext
        if pc is None:
            c = np.average(self.chg[:, :, self.zcut])
        else:
            tmp = np.zeros(zcut_max - zcut_min)
            for ii in range(tmp.size):
                tmp[ii] = np.average(self.chg[:, :, zcut_min + ii])
            c = np.linspace(tmp.min(), tmp.max(), 100)[pc]

        # height of iso-current
        img = np.argmin(np.abs(self.chg[:, :, zcut_min:zcut_max] - c), axis=2)
        img_ext = np.tile(img, self.repeat[::-1]) + self.zcut - zext
        fig, ax = plt.subplots()
        exts = (0, self.a * self.repeat[0], 0, self.b * self.repeat[1])
        plt.xticks([])
        plt.yticks([])
        if self.skew:
            tmp = 90 - self.atoms.lattice.angles[2]
        else:
            tmp = 0

        data = self.get_plot(
            ax, img_ext, exts, mtransforms.Affine2D().skew_deg(tmp, 0),
        )
        plt.savefig(filename)
        plt.close()
        info = {}
        info["img_ext"] = img_ext
        info["data"] = data
        info["scell"] = self.scell
        info["zcut"] = self.zcut
        return info

    def get_plot(self, ax, Z, extent, transform):
        """Apply affine transformation."""
        ax.axis("off")
        im = ax.imshow(
            Z,
            interpolation="none",
            origin="lower",
            extent=extent,
            clip_on=True,
        )  # ,cmap=plt.get_cmap('gray')

        trans_data = transform + ax.transData
        im.set_transform(trans_data)

        # display intended extent of the image
        x1, x2, y1, y2 = im.get_extent()
        data = ax.plot(
            [x1, x2, x2, x1, x1],
            [y1, y1, y2, y2, y1],
            "y--",
            transform=trans_data,
        )
        print("min Z and maxZ", np.min(Z), np.max(Z))
        return data


"""
if __name__ == "__main__":
    plt.switch_backend("agg")
    f = "PARCHG"
    t = TersoffHamannSTM(chg_name=f).constant_height()
    t = TersoffHamannSTM(chg_name=f).constant_current()
"""



"""Helper functions for ML applications."""

from jarvis.db.figshare import data
from sklearn.metrics import mean_absolute_error, r2_score, mean_squared_error
import numpy as np

typical_data_ranges = {
    "formation_energy_peratom": [-5, 5],
    "optb88vdw_bandgap": [0, 10],
    "mbj_bandgap": [0, 10],
    "bulk_modulus_kv": [0, 250],
    "shear_modulus_gv": [0, 250],
    "epsx": [0, 60],
    "epsy": [0, 60],
    "epsz": [0, 60],
    "mepsx": [0, 60],
    "mepsy": [0, 60],
    "mepsz": [0, 60],
    "n-Seebeck": [-600, 10],
    "n-powerfact": [0, 5000],
    "p-Seebeck": [-10, 600],
    "p-powerfact": [0, 5000],
    "slme": [0, 40],
    "spillage": [0, 4],
    "encut": [0, 2000],
    "kpoint_length_unit": [0, 200],
    "dfpt_piezo_max_dielectric": [0, 100],
    "dfpt_piezo_max_dij": [0, 3000],
    "dfpt_piezo_max_eij": [0, 10],
    "ehull": [0, 1],
    "electron_avg_effective_masses_300K": [0, 3],
    "hole_avg_effective_masses_300K": [0, 3],
    "exfoliation_energy": [0, 1000],
    "magmom_oszicar": [0, 10],
    "max_ir_mode": [0, 4000],
    "total_energy_per_atom": [-10, 3],
}


def get_ml_data(ml_property="formation_energy_peratom",
                dataset='cfid_3d', data_ranges=typical_data_ranges):
    """
    Provide arrays/pandas-dataframe as input for ML algorithms.

    Args:

        ml_property: target property to train

        data_ranges: range for filtering data

        dataset: dataset available in jarvis or other array

    Returns:
           X, Y , ids
    """
    import pandas as pd
    if isinstance(dataset, str):
        dataml = data(dataset)
        df = pd.DataFrame(dataml)
    else:
        df = pd.DataFrame(dataset)

    x = []
    y = []
    jid = []
    df2 = df[["desc", "jid", ml_property]].replace("na", np.nan).dropna()
    for ii, i in df2.iterrows():
        if data_ranges is None:
            if (
                len(i["desc"]) == 1557
                and float(i[ml_property]) != float("inf")
                and i[ml_property] != "na"
            ):
                x.append(i["desc"])
                y.append(i[ml_property])
                jid.append(i["jid"])
        else:
            if (
                len(i["desc"]) == 1557
                and float(i[ml_property]) != float("inf")
                and i[ml_property] != "na"
                and float(i[ml_property]) <= data_ranges[ml_property][1]
                and float(i[ml_property]) >= data_ranges[ml_property][0]
            ):
                x.append(i["desc"])
                y.append(i[ml_property])
                jid.append(i["jid"])
    return np.array(x, dtype="float"), np.array(y, dtype="float"), jid


def regr_scores(test, pred):
    """Provide generic regresion scores.

    Args:

       pred: predicted values

       test: held data for testing

    Returns:
        info: with metrics
    """
    rmse = np.sqrt(mean_squared_error(test, pred))
    r2 = r2_score(test, pred)
    mae = mean_absolute_error(test, pred)
    info = {}
    info["mae"] = mae
    info["rmse"] = rmse
    info["r2"] = r2
    info["test"] = test
    info["pred"] = pred
    return info


def binary_class_dat(X=[], Y=[], tol=0.1):
    """
    Categorize a continous dataset in 1/0 with a threshold "tol".

    TODO: replace with OneHotEncoder
    """
    Y1 = []
    for i, j in zip(X, Y):
        if j >= tol:
            Y1.append(1)
        else:
            Y1.append(0)
    return X, Y1



import os

import tarfile

example_fold_tgz = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "examples",
    "vasp",
    "SiOptb88.tgz",
)


example_fold = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "examples",
    "vasp",
    "SiOptb88",
)

if not os.path.isdir(example_fold):
    tar = tarfile.open(example_fold_tgz)
    tar.extractall(example_fold)
    tar.close()




vasp_path = os.path.join(
    os.path.dirname(__file__), "..", "..", "..", "examples", "vasp", "SiOptb88", "SiOptb88"
)

cwd = str(os.getcwd())


def test_vasp():
    os.chdir(vasp_path)
    cmd = str("python master.py")
    os.system(cmd)


os.chdir(cwd)



"""Modules for running LAMMPS calculations."""

from jarvis.analysis.structure.spacegroup import (
    Spacegroup3D,
    symmetrically_distinct_miller_indices,
)
from jarvis.core.atoms import Atoms, get_supercell_dims, ase_to_atoms
from jarvis.io.vasp.inputs import Poscar
from jarvis.io.lammps.inputs import LammpsInput, LammpsData
from jarvis.tasks.lammps.templates.templates import GenericInputs
from jarvis.io.lammps.outputs import analyze_log, read_dump
from jarvis.analysis.defects.vacancy import Vacancy
from jarvis.analysis.defects.surface import Surface
import shutil
import os
import subprocess
import json
from collections import OrderedDict
import numpy as np


class JobFactory(object):
    """Class for generic LAMMPS calculations."""

    def __init__(self, name="", pair_style="", pair_coeff="", control_file=""):
        """
        Use in defining a LAMMPS job.

        With following arguments.
        Args:
            pair_style :  LAMMPS pair_style, e.g. "eam/alloy"

            pair_coeff : path for pair-coefficients file

            control_file :  control-file with units, include modules
                          for running LAMMPS calculation , see examples

            name : generic name
        """
        self.pair_style = pair_style
        self.pair_coeff = pair_coeff
        self.control_file = control_file
        self.name = name

    def all_props_eam_alloy(
        self,
        atoms=None,
        ff_path="",
        lammps_cmd="",
        enforce_conventional_structure=True,
        enforce_c_size=0,
        extend=1,
    ):
        """
        Provide generic function for LAMMPS calculations using eam/alloy.

        Must provide Atoms class and path to force-field.
        Args:
            atoms :  Atoms object

            ff_path :  inter-atomic potential path

            lammps_cmd : LAMMPS executable path

            enforce_conventional_structure :
            whether to enforce conventional cell

            enforce_c_size : minimum cell-sizes

            extend : used for round-off during making supercells
        """
        if enforce_conventional_structure:
            atoms = Spacegroup3D(atoms).conventional_standard_structure

        if enforce_c_size is not None:
            dim = get_supercell_dims(atoms, enforce_c_size=enforce_c_size)
            atoms = atoms.make_supercell([dim[0], dim[1], dim[2]])

        self.pair_style = "eam/alloy"
        self.pair_coeff = ff_path
        parameters = {
            "pair_style": self.pair_style,
            "atom_style": "charge",
            "pair_coeff": self.pair_coeff,
        }
        parameters["control_file"] = "inelast.mod"
        en, final_str, forces = LammpsJob(
            atoms=atoms,
            jobname="ELASTIC",
            parameters=parameters,
            lammps_cmd=lammps_cmd,
        ).runjob()
        print("en, final_str, forces", en, final_str, forces)

        indices = symmetrically_distinct_miller_indices(
            max_index=1, cvn_atoms=atoms
        )
        for i in indices:
            surf = Surface(atoms=final_str, indices=i).make_surface()
            jobname = str("Surf-") + str("_".join(map(str, i)))
            en2, final_str2, forces2 = LammpsJob(
                atoms=surf,
                jobname=jobname,
                parameters=parameters,
                lammps_cmd=lammps_cmd,
            ).runjob()

        # sys.exit()

        v = Vacancy(atoms=final_str).generate_defects(enforce_c_size=5)
        print("vacs=", v)
        for i, ii in enumerate(v):
            jobname = (
                str("symbol-")
                + str(ii._symbol)
                + str("-")
                + str("Wycoff-")
                + str(ii._wyckoff_multiplicity)
            )
            print("ii._defect_structure", ii._atoms)
            en2, final_str2, forces2 = LammpsJob(
                atoms=ii._defect_structure,
                jobname=jobname,
                parameters=parameters,
                lammps_cmd=lammps_cmd,
            ).runjob()

        self.phonons(atoms=atoms, lammps_cmd=lammps_cmd, parameters=parameters)

    # def optimize_and_elastic(self):
    #     pass

    # def surface_energy(self):
    #     pass

    # def vacancy(self):
    #     pass

    # def phonon(self):
    #     pass
    def phonons(
        self, atoms=None, lammps_cmd="", enforce_c_size=15.0, parameters={}
    ):
        """Make Phonon calculation setup."""
        from phonopy import Phonopy
        from phonopy.file_IO import (
            #    parse_FORCE_CONSTANTS,
            write_FORCE_CONSTANTS,
        )

        bulk = atoms.phonopy_converter()

        dim = get_supercell_dims(atoms, enforce_c_size=enforce_c_size)
        atoms = atoms.make_supercell([dim[0], dim[1], dim[2]])

        Poscar(atoms).write_file("POSCAR")

        atoms = atoms.make_supercell_matrix([dim[0], dim[1], dim[2]])
        Poscar(atoms).write_file("POSCAR-Super.vasp")

        phonon = Phonopy(
            bulk, [[dim[0], 0, 0], [0, dim[1], 0], [0, 0, dim[2]]]
        )
        print("[Phonopy] Atomic displacements1:", bulk)
        print(
            "[Phonopy] Atomic displacements2:", phonon, dim[0], dim[1], dim[2]
        )
        phonon.generate_displacements(distance=0.03)
        disps = phonon.get_displacements()
        print("[Phonopy] Atomic displacements3:", disps)
        for d in disps:
            print("[Phonopy]", d[0], d[1:])
        supercells = phonon.get_supercells_with_displacements()

        # Force calculations by calculator
        set_of_forces = []
        disp = 0
        from ase import Atoms as AseAtoms

        for scell in supercells:
            ase_atoms = AseAtoms(
                symbols=scell.get_chemical_symbols(),
                scaled_positions=scell.get_scaled_positions(),
                cell=scell.get_cell(),
                pbc=True,
            )
            j_atoms = ase_to_atoms(ase_atoms)
            disp = disp + 1

            parameters["control_file"] = "run0.mod"
            a, b, forces = LammpsJob(
                atoms=j_atoms,
                lammps_cmd=lammps_cmd,
                parameters=parameters,
                jobname="disp-" + str(disp),
            ).runjob()
            print("forces=", forces)
            drift_force = forces.sum(axis=0)
            print("drift forces=", drift_force)
            # Simple translational invariance
            for force in forces:
                force -= drift_force / forces.shape[0]
            set_of_forces.append(forces)
        phonon.produce_force_constants(forces=set_of_forces)

        write_FORCE_CONSTANTS(
            phonon.get_force_constants(), filename="FORCE_CONSTANTS"
        )
        print()
        print("[Phonopy] Phonon frequencies at Gamma:")


class LammpsJob(object):
    """Construct a class representing a LAMMPS job."""

    def __init__(
        self,
        atoms=None,
        parameters={
            "pair_style": "eam/alloy",
            "pair_coeff": "abc.alloy",
            "atom_style": "charge",
            "control_file": "inelast.mod",
        },
        lammps_cmd=None,
        output_file="lammps.out",
        stderr_file="std_err.txt",
        jobname="testt",
        attempts=5,
        copy_files=[],
        element_order=[],
    ):
        """
        Use for defining a LAMMPS job.

        Provide follwoing arguments.

        Args:
            atoms :  Atoms object

            element_order :
            element order used in accessing force-field parameters

            parameters :  LAMMPS input parameter dictionary

            lammps_cmd : LAMMPS executable path

            output_file :  standard output file

            stderr_file :  standard error file

            jobname :  Job name

            attempts :  number of attempts before crashing the job, TODO

            copy_files : copy certain files before a job
        """
        self.atoms = atoms
        self.element_order = element_order
        self.parameters = parameters
        self.lammps_cmd = lammps_cmd
        self.output_file = output_file
        self.stderr_file = stderr_file
        self.jobname = jobname
        self.attempts = attempts
        self.copy_files = copy_files

    def to_dict(self):
        """Convert to a dictionary."""
        d = OrderedDict()
        d["atoms"] = self.atoms.to_dict()
        d["element_order"] = self.element_order
        d["parameters"] = self.parameters
        d["lammps_cmd"] = self.lammps_cmd
        d["output_file"] = self.output_file
        d["stderr_file"] = self.stderr_file
        d["jobname"] = self.jobname
        d["attempts"] = self.attempts
        d["copy_files"] = self.copy_files
        return d

    @classmethod
    def from_dict(self, d={}):
        """Load from a dictionary."""
        return LammpsJob(
            atoms=Atoms.from_dict(d["atoms"]),
            element_order=self.element_order,
            parameters=d["parameters"],
            lammps_cmd=d["lammps_cmd"],
            output_file=d["output_file"],
            stderr_file=d["stderr_file"],
            jobname=d["jobname"],
            attempts=d["attempts"],
            copy_files=d["copy_files"],
        )

    def write_input(self):
        """Write LAMMPS input files."""
        lmp = LammpsData().atoms_to_lammps(atoms=self.atoms)
        self.element_order = lmp._element_order
        lmp.write_file("data")
        LammpsInput(LammpsDataObj=lmp).write_lammps_in(
            parameters=self.parameters
        )
        for i in self.copy_files:
            shutil.copy2(i, ".")
        if "control_file" in self.parameters:
            if self.parameters["control_file"] == "inelast.mod":
                GenericInputs().elastic_general(path=".")
            if self.parameters["control_file"] == "run0.mod":
                GenericInputs().run0(path=".")

    def run(self):
        """Run a job with subprocess."""
        with open(self.output_file, "w") as f_std, open(
            self.stderr_file, "w", buffering=1
        ) as f_err:
            p = subprocess.Popen(
                self.lammps_cmd, shell=True, stdout=f_std, stderr=f_err
            )
            p.wait()
        return p

    def runjob(self):
        """Constrct  a generic LAMMPS job submission."""
        attempt = 0
        wait = False
        while not wait:
            attempt = attempt + 1
            if attempt == self.attempts:
                wait = True
            folder = str(os.getcwd()) + str("/") + str(self.jobname)
            if not os.path.exists(folder):
                os.makedirs(folder)
            os.chdir(folder)

            if os.path.isfile("./log.lammps"):
                try:

                    (
                        en,
                        press,
                        toten,
                        c11,
                        c22,
                        c33,
                        c12,
                        c13,
                        c23,
                        c44,
                        c55,
                        c66,
                        c14,
                        c16,
                        c24,
                        c25,
                        c26,
                        c34,
                        c35,
                        c36,
                        c45,
                        c46,
                        c56,
                    ) = analyze_log("./log.lammps")
                    wait = True
                except Exception:
                    pass
                # print ('toten',toten)
            else:
                self.write_input()
                self.run()
                try:

                    (
                        en,
                        press,
                        toten,
                        c11,
                        c22,
                        c33,
                        c12,
                        c13,
                        c23,
                        c44,
                        c55,
                        c66,
                        c14,
                        c16,
                        c24,
                        c25,
                        c26,
                        c34,
                        c35,
                        c36,
                        c45,
                        c46,
                        c56,
                    ) = analyze_log("./log.lammps")
                    wait = True
                except Exception:
                    pass
            pot = os.path.join(os.getcwd(), "potential.mod")
            # print ('toten2',toten,pot)
            initial_str = LammpsData().read_data(
                filename="data",
                element_order=self.element_order,
                potential_file=pot,
            )
            final_str = LammpsData().read_data(
                potential_file=pot,
                filename="data0",
                element_order=self.element_order,
            )
            forces = []
            try:
                forces = read_dump(data="0.dump")
            except Exception:
                pass
            forces = np.array(forces)
            data_cal = []
            data_cal.append(
                {
                    "jobname": self.jobname,
                    "initial_pos": initial_str.to_dict(),
                    "pair_style": str(self.parameters["pair_style"]),
                    "pair_coeff": str(self.parameters["pair_coeff"]),
                    "final_energy": float(toten),
                    "en": en,
                    "press": press,
                    "final_str": final_str.to_dict(),
                }
            )

            json_file = str(self.jobname) + str(".json")
            os.chdir("../")
            f_json = open(json_file, "w")
            f_json.write(json.dumps(data_cal))
            f_json.close()
            return en, final_str, forces



"""Class for analyzing  WT.out file."""

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec

plt.switch_backend("agg")


class WTOut(object):
    """Construct WT.out related object."""

    def __init__(self, path=""):
        """Initialize with path to WT.out file."""
        self.path = path

    def get_z2_index(self):
        """Get Z2 index."""
        f = open(self.path, "r")
        lines = f.read().splitlines()
        f.close()
        strng = 0
        for ii, i in enumerate(lines):
            if "# z2 number for 6 planes" in i:
                for j in range(6):
                    tmp = lines[ii + j + 1].split()

                    if (
                        tmp[0] == "k1=0.0,"
                        or tmp[0] == "k2=0.0,"
                        or tmp[0] == "k3=0.0,"
                        or tmp[0] == "k3=0.5,"
                        or tmp[0] == "k2=0.5,"
                        or tmp[0] == "k1=0.5,"
                    ):
                        val = tmp[3]
                        strng = strng + float(val)
                    if tmp[0] == "k1=0.5,":
                        weak1 = tmp[3]
                    if tmp[0] == "k2=0.5,":
                        weak2 = tmp[3]
                    if tmp[0] == "k3=0.5,":
                        weak3 = tmp[3]
        index = (
            str(int(strng % 2))
            + str(";")
            + str(weak1)
            + str(",")
            + str(weak2)
            + str(",")
            + str(weak3)
        )
        return index

    def get_chern_number(self):
        """Get Chern index."""
        f = open(self.path, "r")
        lines = f.read().splitlines()
        f.close()
        chrn = []
        try:
            for j, jj in enumerate(lines):
                if "# Chern number for 6 planes" in jj:
                    for k in range(1, 7):
                        tmp = float(lines[j + k].split(":")[1])
                        if tmp not in chrn:
                            chrn.append(tmp)
        except Exception:
            pass
        return chrn


def parse_chern_dat(
    chern_dat="wanniercenter3D_Chern.dat", filename="mychern.png"
):
    """Plot wanniercenter3D_Chern.dat file."""
    x = np.loadtxt(chern_dat)
    if filename is not None:
        the_grid = GridSpec(3, 2)
        plt.rcParams.update({"font.size": 18})
        plt.figure(figsize=(12, 12))

        plt.subplot(the_grid[0, 0])
        plt.title("(a) k$_1$=0.0")
        plt.xlabel("k$_2$")
        plt.plot(x[:, 0], x[:, 1], ".")
        plt.subplot(the_grid[0, 1])
        plt.title("(b) k$_1$=0.5")
        plt.xlabel("k$_2$")
        plt.plot(x[:, 0], x[:, 2], ".")

        plt.subplot(the_grid[1, 0])
        plt.xlabel("k$_1$")
        plt.title("(c) k$_2$=0.0")
        plt.plot(x[:, 0], x[:, 3], ".")
        plt.subplot(the_grid[1, 1])
        plt.xlabel("k$_1$")
        plt.title("(d) k$_2$=0.5")
        plt.plot(x[:, 0], x[:, 4], ".")

        plt.subplot(the_grid[2, 0])
        plt.title("(e) k$_3$=0.0")
        plt.xlabel("k$_2$")
        plt.plot(x[:, 0], x[:, 5], ".")
        plt.subplot(the_grid[2, 1])
        plt.title("(f) k$_3$=0.5")
        plt.xlabel("k$_2$")
        plt.plot(x[:, 0], x[:, 6], ".")
        plt.tight_layout()
        plt.savefig(filename)
        plt.close()
    return x


def parse_nodes_dat(fname="Nodes.dat"):
    """Parse Nodedat file."""
    f = open(fname, "r")
    lines = f.read().splitlines()
    f.close()
    nodes = []
    for i in lines:
        if "#" not in i:
            nodes.append(i.split())
    nodes = np.array(nodes, dtype="float")
    return nodes


"""
if __name__ == "__main__":
    wt = "WT.out"
    z2 = WTOut(path=wt).get_z2_index()
    print(z2)
    chrn = WTOut(path=wt).get_chern_number()
    print(chrn)
"""



from jarvis.analysis.elastic.tensor import ElasticTensor
import os, tarfile
from jarvis.io.lammps.outputs import parse_folder
from jarvis.io.vasp.outputs import Vasprun, Outcar

example_fold_tgz = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "..",
    "examples",
    "vasp",
    "SiOptb88.tgz",
)


example_fold = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "..",
    "examples",
    "vasp",
    "SiOptb88",
)

if not os.path.isdir(example_fold):
    tar = tarfile.open(example_fold_tgz)
    tar.extractall(example_fold)
    tar.close()


outcar = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "..",
    "examples",
    "vasp",
    "SiOptb88",
    "SiOptb88",
    "MAIN-ELASTIC-bulk@mp_149",
    "OUTCAR",
)


def test_vasp_et():
    out = Outcar(outcar)
    et = ElasticTensor(out.elastic_props()["cij"])
    print(et.to_dict())


example_fold_tgz = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "..",
    "examples",
    "lammps",
    "Al03.eam.alloy_nist.tgz",
)


example_fold = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "..",
    "examples",
    "lammps",
    "Al03.eam.alloy_nist",
)

if not os.path.isdir(example_fold):
    tar = tarfile.open(example_fold_tgz)
    tar.extractall(example_fold)
    tar.close()


fold_lammps = os.path.join(
    os.path.dirname(__file__),
    "..",
    "..",
    "..",
    "..",
    "examples",
    "lammps",
    "Al03.eam.alloy_nist",
    "Al03.eam.alloy_nist",
    "bulk@mp-134_fold",
    "bulk@mp-134",
)


def test_lammps_et():

    data = parse_folder(fold_lammps)
    print(data["elastic_tensor"]["raw_et_tensor"])


# test_lammps_et()



"""Function to analze LAMMPS output."""
import numpy as np
import glob
import os
from jarvis.analysis.elastic.tensor import ElasticTensor
from jarvis.io.lammps.inputs import LammpsData
from jarvis.io.phonopy.outputs import bandstructure_plot, total_dos


def parse_potential_mod(mod="potential.mod"):
    """Parse potentials.mod input file."""
    info = {}
    f = open(mod, "r")
    lines = f.read().splitlines()
    f.close()
    for i in lines:
        if "pair_style" in i:
            pair_style = i.split("pair_style")[1]
        if "pair_coeff" in i:
            pair_coeff = i.split()[3].split("/")[-1]
            elements = i.split()[4:]

    info["pair_style"] = pair_style
    info["pair_coeff"] = pair_coeff
    info["elements"] = elements
    return info


def read_data(data=None, ff=None, element_order=[]):
    """
    Read LAMMPS data file.

    Args:
        data: data file path

        ff: potential.mod/potential information file path

    Returns:
          Atoms object
    """
    lmp_data = LammpsData()
    return lmp_data.read_data(
        filename=data,
        element_order=element_order,
        potential_file=ff,
        verbose=False,
    )


def parse_log(log="log.lammps"):
    """
    Analyzes log.lammps file.

    Please note, the output format heavily depends on the input file
    A generic input is taken here.
    Args:
        log: path to log.lammps file

    Returns:
          en: energy/atom

          press: pressure

          toten: total energy

          cij: elastic constants
    """
    en = 0
    press = 0
    c11 = 0
    c22 = 0
    c33 = 0
    c44 = 0
    c55 = 0
    c66 = 0
    c12 = 0
    c13 = 0
    c23 = 0
    c14 = 0
    c15 = 0
    c16 = 0
    c14 = 0
    c24 = 0
    c25 = 0
    c26 = 0
    c34 = 0
    c35 = 0
    c36 = 0
    c45 = 0
    c46 = 0
    c56 = 0
    Et = ""
    info = {}
    try:
        logfile = open(log, "r")
        lines = logfile.read().splitlines()
        for i, line in enumerate(lines):
            if "Loop time of" in line:
                # toten = float(lines[i - 1].split()[12])
                press = float(lines[i - 1].split()[2])
                press = float(press) * 0.0001
                denom = float(lines[i - 1].split()[17])
                en = float(lines[i - 1].split()[12]) / denom
                break
        logfile.close()
    except Exception:
        print("Cannot parse energy and pressure information.", log)
        pass

    try:
        logfile = open(log, "r")
        lines = logfile.read().splitlines()
        for i, line in enumerate(lines):
            if 'print "Elastic Constant C11all = ${C11all} ${cunits}"' in line:
                c11 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C22all = ${C22all} ${cunits}"' in line:
                c22 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C33all = ${C33all} ${cunits}"' in line:
                c33 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C12all = ${C12all} ${cunits}"' in line:
                c12 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C13all = ${C13all} ${cunits}"' in line:
                c13 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C23all = ${C23all} ${cunits}"' in line:
                c23 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C44all = ${C44all} ${cunits}"' in line:
                c44 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C55all = ${C55all} ${cunits}"' in line:
                c55 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C66all = ${C66all} ${cunits}"' in line:
                c66 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C14all = ${C14all} ${cunits}"' in line:
                c14 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C15all = ${C15all} ${cunits}"' in line:
                c15 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C16all = ${C16all} ${cunits}"' in line:
                c16 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C24all = ${C24all} ${cunits}"' in line:
                c24 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C25all = ${C25all} ${cunits}"' in line:
                c25 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C26all = ${C26all} ${cunits}"' in line:
                c26 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C34all = ${C34all} ${cunits}"' in line:
                c34 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C35all = ${C35all} ${cunits}"' in line:
                c35 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C36all = ${C36all} ${cunits}"' in line:
                c36 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C45all = ${C45all} ${cunits}"' in line:
                c45 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C46all = ${C46all} ${cunits}"' in line:
                c46 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C56all = ${C56all} ${cunits}"' in line:
                c56 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
        logfile.close()
        cij = np.array(
            [
                [c11, c12, c13, c14, c15, c16],
                [c12, c22, c23, c24, c25, c26],
                [c13, c23, c33, c34, c35, c36],
                [c14, c24, c34, c44, c45, c46],
                [c15, c25, c35, c45, c55, c56],
                [c16, c26, c36, c46, c56, c66],
            ],
            dtype="float",
        )
        Et = ElasticTensor(et_tensor=cij).to_dict()
    except Exception:
        print("Cannot parse log.lammps", log)
        pass
    info["energy_per_atom"] = en
    info["system_pressure"] = press
    info["elastic_tensor"] = Et
    return info


def get_chem_pot(all_json_data={}):
    """Get minumum energy for each elemental specie."""
    all_possible_species = []
    elemental_energies = {}
    for i, j in all_json_data.items():
        #  print (i)
        if (
            i != "jid"
            and i != "source_folder"
            and i != "bulk_data"
            and i != "bulk_energy_per_atom"
        ):
            for el in j["final_str"].elements:
                if el not in all_possible_species:
                    all_possible_species.append(el)
            if len(j["final_str"].uniq_species) == 1:
                tmp_element = j["final_str"].uniq_species[0]
                tmp_energy = j["energy_per_atom"]
                elemental_energies.setdefault(tmp_element, []).append(
                    [tmp_energy, i]
                )
    chem_pot = {}
    for i, j in elemental_energies.items():
        chem_pot[i] = sorted(j, key=lambda x: x[0])[0][0]
    if len(all_possible_species) != len(elemental_energies.keys()):
        raise ValueError(
            "Error: Chemical potential for all elements are not available"
        )
        raise ValueError(all_possible_species, elemental_energies.keys())
    return chem_pot


def parse_folder(path="bulk@mp-1487_fold/bulk@mp-1487",):
    """Parse individual LAMMPS run."""
    info = {}
    ff = os.path.join(path, "potential.mod")
    pot = parse_potential_mod(ff)
    initial_str = read_data(data=os.path.join(path, "data"), ff=ff)
    final_str = read_data(data=os.path.join(path, "data0"), ff=ff)
    log_path = os.path.join(path, "log.lammps")
    pair_style = pot["pair_style"]
    pair_coeff = pot["pair_coeff"]
    info["pair_style"] = pair_style
    info["pair_coeff"] = pair_coeff
    info["initial_str"] = initial_str
    info["final_str"] = final_str

    dat = parse_log(log_path)
    info["energy_per_atom"] = dat["energy_per_atom"]
    info["system_pressure"] = dat["system_pressure"]
    info["elastic_tensor"] = dat["elastic_tensor"]

    return info


def parse_material_calculation_folder(path="bulk@mp-1487_fold", jid="x"):
    """
    Parse individual LAMMPS material run.

    with optimization, vacancy, phonon, surface etc.
    """
    cwd = os.getcwd()
    jid_file = os.path.join(path, "JARVISFF-ID")
    if os.path.exists(jid_file):
        f = open(jid_file, "r")
        lines = f.read().splitlines()
        f.close()
        jid = lines[0]
    print(path)
    info = {}
    info["jid"] = jid
    info["source_folder"] = path
    bulk_energy_per_atom = ""
    for i in glob.glob(path + "/*.json"):
        try:
            json_file_name = i.split("/")[-1]
            json_file_path = i.split(".json")[0]
            print("json_file_name", json_file_name)
            print("json_file_path", json_file_path)
            fold_path = os.path.join(path, json_file_path)
            tmp_info = parse_folder(fold_path)
            info[json_file_name] = tmp_info
            if (
                "bulk" in json_file_name
                and "cellmax" not in json_file_name
                and "sbulk" not in json_file_name
            ):
                bulk_energy_per_atom = tmp_info["energy_per_atom"]
                info["bulk_data"] = tmp_info
        except Exception:
            print("Error cannot parse bulk file.")
            pass
    info["bulk_energy_per_atom"] = bulk_energy_per_atom
    # Each element has energy_per_atom,
    # system_pressure,elastic_tensor,final_str,initial_str
    print("Found", len(info.keys()), "folders")
    try:
        chem_pot = get_chem_pot(info)
        info["chem_pot"] = chem_pot
    except Exception:
        print("Seems like didnt run vacancy calcs.")
        pass
    # print (chem_pot)
    vacancy_info = []
    surface_info = []
    try:
        for i, j in info.items():
            if "vacancy" in i:
                try:
                    element_vacant = i.split("-")[-1].split("@")[0]
                    perfect_energy = (
                        j["final_str"].num_atoms + 1
                    ) * bulk_energy_per_atom
                    defect_energy = (
                        j["final_str"].num_atoms * j["energy_per_atom"]
                    )
                    mu = chem_pot[element_vacant]
                    Ef = defect_energy - perfect_energy + mu
                    mult = i.split("mult-")[1].split("_")[0]
                    vacancy_info.append([element_vacant, mult, Ef])
                    # print ('Ef',i,element_vacant, Ef,mult)
                except Exception:
                    print("Error parsing vacancy.", i)
                    pass
            if "Surf" in i:
                try:
                    perfect_energy = (
                        j["final_str"].num_atoms
                    ) * bulk_energy_per_atom
                    defect_energy = (
                        j["final_str"].num_atoms * j["energy_per_atom"]
                    )
                    m = j["final_str"].lattice.matrix
                    area = np.linalg.norm(np.cross(m[0], m[1]))
                    Ef = 16 * (defect_energy - perfect_energy) / (2 * area)
                    surf_name = i.split("@")[0].split("-")[1].replace("_", " ")
                    surface_info.append([surf_name, Ef])
                except Exception:
                    print("Error parsing surface.", i)
                    pass
                # print (i,Ef)
    except Exception:
        print("Error parsing vacancy-surface.")
        pass
    # print ('JID',jid)
    info["surface_info"] = surface_info
    info["vacancy_info"] = vacancy_info
    dos_file = os.path.join(path, "Phonon", "total_dos.dat")
    # mesh_file = os.path.join(path, "Phonon", "mesh.yaml")
    band_file = os.path.join(path, "Phonon", "band.yaml")
    if os.path.exists(dos_file):
        dos_freq, dos_intensity = total_dos(tot_dos=dos_file)
        info["dos_freq"] = dos_freq
        info["dos_intensity"] = dos_intensity
    if os.path.exists(band_file):
        (
            band_frequencies,
            band_distances,
            band_labels,
            band_label_points,
        ) = bandstructure_plot(band_file)
        info["band_frequencies"] = band_frequencies
        info["band_distances"] = band_distances
        info["band_labels"] = band_labels
        info["band_label_points"] = band_label_points
    os.chdir(cwd)
    return info


def parse_full_ff_folder(path="Mishin-Ni-Al-2009.eam.alloy_nist",):
    """Parse complete FF calculation folder."""
    cwd = os.getcwd()
    os.chdir(path)
    print("path", path)
    tmp_path = path + "/*_fold"
    for i in glob.glob(tmp_path):
        print(i)
        tmp_fold = os.path.join(path, i)
        info = parse_material_calculation_folder(tmp_fold)
    os.chdir(cwd)
    return info


def analyze_log(log="log.lammps"):
    """
    Analyzes log.lammps file.

    Please note, the output format heavily depends on the input file
    A generic input is taken here.
    Args:
        log: path to log.lammps file
    Returns:
          en: energy/atom
          press: pressure
          toten: total energy
          cij: elastic constants
    """
    en = 0
    press = 0
    c11 = 0
    c22 = 0
    c33 = 0
    c44 = 0
    c55 = 0
    c66 = 0
    c12 = 0
    c13 = 0
    c23 = 0
    c14 = 0
    # c15 = 0
    c16 = 0
    c14 = 0
    c24 = 0
    c25 = 0
    c26 = 0
    c34 = 0
    c35 = 0
    c36 = 0
    c45 = 0
    c46 = 0
    c56 = 0
    try:
        logfile = open(log, "r")
        lines = logfile.read().splitlines()
        for i, line in enumerate(lines):
            if "Loop time of" in line:
                toten = float(lines[i - 1].split()[12])
                press = float(lines[i - 1].split()[2])
                press = float(press) * 0.0001
                denom = float(lines[i - 1].split()[17])
                en = float(lines[i - 1].split()[12]) / denom
                break
        logfile.close()
    except Exception:
        pass
    try:
        logfile = open(log, "r")
        lines = logfile.read().splitlines()
        for i, line in enumerate(lines):
            if 'print "Elastic Constant C11all = ${C11all} ${cunits}"' in line:
                c11 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C22all = ${C22all} ${cunits}"' in line:
                c22 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C33all = ${C33all} ${cunits}"' in line:
                c33 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C12all = ${C12all} ${cunits}"' in line:
                c12 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C13all = ${C13all} ${cunits}"' in line:
                c13 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C23all = ${C23all} ${cunits}"' in line:
                c23 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C44all = ${C44all} ${cunits}"' in line:
                c44 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C55all = ${C55all} ${cunits}"' in line:
                c55 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C66all = ${C66all} ${cunits}"' in line:
                c66 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C14all = ${C14all} ${cunits}"' in line:
                c14 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C16all = ${C16all} ${cunits}"' in line:
                c16 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C24all = ${C24all} ${cunits}"' in line:
                c24 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C25all = ${C25all} ${cunits}"' in line:
                c25 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C26all = ${C26all} ${cunits}"' in line:
                c26 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C34all = ${C34all} ${cunits}"' in line:
                c34 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C35all = ${C35all} ${cunits}"' in line:
                c35 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C36all = ${C36all} ${cunits}"' in line:
                c36 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C45all = ${C45all} ${cunits}"' in line:
                c45 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C46all = ${C46all} ${cunits}"' in line:
                c46 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C56all = ${C56all} ${cunits}"' in line:
                c56 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
        logfile.close()
    except Exception:
        pass
    return (
        round(en, 2),
        round(press, 2),
        float(toten),
        round(float(c11), 1),
        round(float(c22), 1),
        round(float(c33), 1),
        round(float(c12), 1),
        round(float(c13), 1),
        round(float(c23), 1),
        round(float(c44), 1),
        round(float(c55), 1),
        round(float(c66), 1),
        round(float(c14), 1),
        # round(float(c15), 1),
        round(float(c16), 1),
        round(float(c24), 1),
        round(float(c25), 1),
        round(float(c26), 1),
        round(float(c34), 1),
        round(float(c35), 1),
        round(float(c36), 1),
        round(float(c45), 1),
        round(float(c46), 1),
        round(float(c56), 1),
    )


def read_dump(data=None):
    """Read LAMMPS dump file."""
    f = open(data, "r")
    lines = f.read().splitlines()
    for i, line in enumerate(lines):
        if "NUMBER OF ATOMS" in line:
            natoms = int(lines[i + 1].split()[0])
    x = np.zeros((natoms))
    y = np.zeros((natoms))
    z = np.zeros((natoms))
    coords = list()  # np.zeros((natoms))
    for i, line in enumerate(lines):
        if "ITEM: ATOMS" in line:
            for j in range(0, natoms):
                x[j] = (lines[i + j + 1]).split()[1]
                y[j] = (lines[i + j + 1]).split()[2]
                z[j] = (lines[i + j + 1]).split()[3]
                coords.append([x[j], y[j], z[j]])
    f.close()
    prop = np.asarray(coords)
    return prop


# p=read_data()
# print (p)
# parse_potential_mod()
"""
if __name__ == "__main__":
    lg = "log.lammps"
    x = analyze_log(lg)
    print(x)
"""



from jarvis.tasks.vasp.vasp import VaspJob, write_jobfact_optb88vdw
from jarvis.io.vasp.inputs import Potcar, Incar, Poscar
from jarvis.db.jsonutils import dumpjson
from jarvis.core.atoms import Atoms
from jarvis.core.kpoints import Kpoints3D
from jarvis.tasks.queue_jobs import Queue
import os
from jarvis.tasks.vasp.vasp import JobFactory, write_jobfact_optb88vdw
from jarvis.db.figshare import get_jid_data


tmp_dict = get_jid_data(jid="JVASP-816", dataset="dft_3d")["atoms"]
atoms = Atoms.from_dict(tmp_dict)


vasp_cmd = "mpirun /users/knc6/VASP/vasp54/src/vasp.5.4.1DobbySOC2/bin/vasp_std"
copy_files = ["/users/knc6/bin/vdw_kernel.bindat"]
submit_cmd = ["qsub",  "submit_job"]
jids = ["JVASP-1002", "JVASP-1067"]

for jid in jids:
    d = get_jid_data(jid=jid, dataset="dft_3d")
    atoms = Atoms.from_dict(d["atoms"]).get_primitive_atoms
    mat = Poscar(atoms)
    mat.comment = "bulk@" + str(jid)
    cwd_home = os.getcwd()
    dir_name = d["jid"] + "_" + str("PBEBO")
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    os.chdir(dir_name)
    job = JobFactory(vasp_cmd=vasp_cmd, poscar=mat, copy_files=copy_files,)
    dumpjson(data=job.to_dict(), filename="job_fact.json")
    write_jobfact_optb88vdw(pyname="job_fact.py", job_json="job_fact.json")

    # Example job commands, need to change based on your cluster
    job_line = (
        "source ~/anaconda2/envs/my_jarvis/bin/activate my_jarvis \n"
        + "python job_fact.py"
    )
    name = jid
    directory = os.getcwd()
    Queue.pbs(
        job_line=job_line,
        jobname=name,
        directory=directory,
        submit_cmd=submit_cmd,
    )
    os.chdir(cwd_home)




from jarvis.core.specie import Specie


def test_sp():
    el = Specie("Al")
    assert (
        el.Z,
        round(el.atomic_mass, 2),
        el.symbol,
        round(el.get_chgdescrp_arr[1], 2),
        round(el.get_descrp_arr[1], 2),
    ) == (13, 26.98, "Al", 12.17, 2792.11)



from jarvis.core.lattice import Lattice
import numpy as np
from jarvis.db.figshare import data
from jarvis.core.atoms import Atoms

def test_lat():
    box = [[10, 0, 0], [0, 10, 0], [0, 0, 10]]
    lat = Lattice(box)
    td = lat.to_dict()
    fd = Lattice.from_dict(td)
    frac_coords = [[0, 0, 0], [0.5, 0.5, 0.5]]
    cart_coords = [[0, 0, 0], [5, 5, 5]]
    lll = lat._calculate_lll()
    # print ('lll',lll[0][0][0])
    lll_red = lat.get_lll_reduced_lattice()._lat
    # print("lll_educed", lat.get_lll_reduced_lattice()._lat[0][0])
    assert (
        lat.lat_lengths(),
        lat.lat_angles(),
        round(lat.inv_lattice()[0][0], 2),
        [round(i, 2) for i in lat.lat_angles(radians=True)],
        lat.cart_coords(frac_coords)[1][1],
        lat.frac_coords(cart_coords)[1][1],
        lat.volume,
        lat.parameters,
        lll[0][0][0],
        lll_red[0][0],
    ) == (
        [10.0, 10.0, 10.0],
        [90.0, 90.0, 90.0],
        0.1,
        [1.57, 1.57, 1.57],
        5.0,
        0.5,
        1000.0,
        [10.0, 10.0, 10.0, 90.0, 90.0, 90.0],
        10.0,
        10.0,
    )
    d=data('dft_3d')
    for i in d:
      if i['jid'] == 'JVASP-588':
         atoms=Atoms.from_dict(i['atoms'])
         lll = atoms.lattice._calculate_lll()
         assert lll[1][0][0] == -1

# test_lat()



"""Design interface using Zur algorithm and Anderson rule."""

from itertools import product
import numpy as np
from jarvis.core.atoms import add_atoms, fix_pbc
from jarvis.core.lattice import Lattice


class ZSLGenerator(object):
    """
    Uses Zur algorithm to find best matched interfaces.

    This class is modified from pymatgen.
    """

    def __init__(
        self,
        max_area_ratio_tol=0.09,
        max_area=400,
        max_length_tol=0.03,
        max_angle_tol=0.01,
    ):
        """
        Intialize for a specific film and substrate.

        Parameters for the class.
        Args:
            max_area_ratio_tol(float): Max tolerance on ratio of
                super-lattices to consider equal

            max_area(float): max super lattice area to generate in search

            max_length_tol: maximum length tolerance in checking if two
                vectors are of nearly the same length

            max_angle_tol: maximum angle tolerance in checking of two sets
                of vectors have nearly the same angle between them
        """
        self.max_area_ratio_tol = max_area_ratio_tol
        self.max_area = max_area
        self.max_length_tol = max_length_tol
        self.max_angle_tol = max_angle_tol

    def is_same_vectors(self, vec_set1, vec_set2):
        """
        Check two sets of vectors are the same.

        Args:
            vec_set1(array[array]): an array of two vectors

            vec_set2(array[array]): second array of two vectors
        """
        if (
            np.absolute(rel_strain(vec_set1[0], vec_set2[0]))
            > self.max_length_tol
        ):
            return False
        elif (
            np.absolute(rel_strain(vec_set1[1], vec_set2[1]))
            > self.max_length_tol
        ):
            return False
        elif np.absolute(rel_angle(vec_set1, vec_set2)) > self.max_angle_tol:
            return False
        else:
            return True

    def generate_sl_transformation_sets(self, film_area, substrate_area):
        """Generate transformation sets for film/substrate.

        The transformation sets map the film and substrate unit cells to super
        lattices with a maximum area.

        Args:

            film_area(int): the unit cell area for the film.

            substrate_area(int): the unit cell area for the substrate.

        Returns:
            transformation_sets: a set of transformation_sets defined as:
                1.) the transformation matricies for the film to create a
                super lattice of area i*film area
                2.) the tranformation matricies for the substrate to create
                a super lattice of area j*film area
        """
        transformation_indicies = [
            (i, j)
            for i in range(1, int(self.max_area / film_area))
            for j in range(1, int(self.max_area / substrate_area))
            if np.absolute(film_area / substrate_area - float(j) / i)
            < self.max_area_ratio_tol
        ]

        # Sort sets by the square of the matching area and yield in order
        # from smallest to largest
        for x in sorted(transformation_indicies, key=lambda x: x[0] * x[1]):
            yield (
                gen_sl_transform_matricies(x[0]),
                gen_sl_transform_matricies(x[1]),
            )

    def get_equiv_transformations(
        self, transformation_sets, film_vectors, substrate_vectors
    ):
        """
        Apply the transformation_sets to the film and substrate vectors.

        Generate super-lattices and checks if they matches.
        Returns all matching vectors sets.

        Args:
            transformation_sets(array): an array of transformation sets:
                each transformation set is an array with the (i,j)
                indicating the area multipes of the film and subtrate it
                corresponds to, an array with all possible transformations
                for the film area multiple i and another array for the
                substrate area multiple j.

            film_vectors(array): film vectors to generate super lattices.

            substrate_vectors(array): substrate vectors to generate super
                lattices
        """
        for (
            film_transformations,
            substrate_transformations,
        ) in transformation_sets:
            # Apply transformations and reduce using Zur reduce methodology
            films = [
                reduce_vectors(*np.dot(f, film_vectors))
                for f in film_transformations
            ]

            substrates = [
                reduce_vectors(*np.dot(s, substrate_vectors))
                for s in substrate_transformations
            ]

            # Check if equivalant super lattices
            for (f_trans, s_trans), (f, s) in zip(
                product(film_transformations, substrate_transformations),
                product(films, substrates),
            ):
                if self.is_same_vectors(f, s):
                    yield [f, s, f_trans, s_trans]

    def __call__(self, film_vectors, substrate_vectors, lowest=False):
        """Run the ZSL algorithm to generate all possible matching."""
        film_area = vec_area(*film_vectors)
        substrate_area = vec_area(*substrate_vectors)

        # Generate all super lattice comnbinations for a given set of miller
        # indicies
        transformation_sets = self.generate_sl_transformation_sets(
            film_area, substrate_area
        )

        # Check each super-lattice pair to see if they match
        for match in self.get_equiv_transformations(
            transformation_sets, film_vectors, substrate_vectors
        ):
            # Yield the match area, the miller indicies,
            yield self.match_as_dict(
                match[0],
                match[1],
                film_vectors,
                substrate_vectors,
                vec_area(*match[0]),
                match[2],
                match[3],
            )

            if lowest:
                break

    def match_as_dict(
        self,
        film_sl_vectors,
        substrate_sl_vectors,
        film_vectors,
        substrate_vectors,
        match_area,
        film_transformation,
        substrate_transformation,
    ):
        """
        Return dict which contains ZSL match.

        Args:
            film_miller(array)

            substrate_miller(array)
        """
        d = {}
        d["film_sl_vecs"] = np.asarray(film_sl_vectors)
        d["sub_sl_vecs"] = np.asarray(substrate_sl_vectors)
        d["match_area"] = match_area
        d["film_vecs"] = np.asarray(film_vectors)
        d["sub_vecs"] = np.asarray(substrate_vectors)
        d["film_transformation"] = np.asarray(film_transformation)
        d["substrate_transformation"] = np.asarray(substrate_transformation)

        return d


def gen_sl_transform_matricies(area_multiple):
    """
    Generate the transformation matricies.

    Convert a set of 2D vectors into a super
    lattice of integer area multiple as proven
    in Cassels:
    Cassels, John William Scott. An introduction to the geometry of
    numbers. Springer Science & Business Media, 2012.

    Args:
        area_multiple(int): integer multiple of unit cell area for super
        lattice area.

    Returns:
        matrix_list: transformation matricies to covert unit vectors to
        super lattice vectors.
    """
    return [
        np.array(((i, j), (0, area_multiple / i)))
        for i in get_factors(area_multiple)
        for j in range(area_multiple // i)
    ]


def rel_strain(vec1, vec2):
    """Calculate relative strain between two vectors."""
    return fast_norm(vec2) / fast_norm(vec1) - 1


def rel_angle(vec_set1, vec_set2):
    """
    Calculate the relative angle between two vector sets.

    Args:
        vec_set1(array[array]): an array of two vectors.

        vec_set2(array[array]): second array of two vectors.
    """
    return (
        vec_angle(vec_set2[0], vec_set2[1])
        / vec_angle(vec_set1[0], vec_set1[1])
        - 1
    )


def fast_norm(a):
    """Much faster variant of numpy linalg norm."""
    return np.sqrt(np.dot(a, a))


def vec_angle(a, b):
    """Calculate angle between two vectors."""
    cosang = np.dot(a, b)
    sinang = fast_norm(np.cross(a, b))
    return np.arctan2(sinang, cosang)


def vec_area(a, b):
    """Area of lattice plane defined by two vectors."""
    return fast_norm(np.cross(a, b))


def reduce_vectors(a, b):
    """Generate independent and unique basis vectors based on Zur et al."""
    if np.dot(a, b) < 0:
        return reduce_vectors(a, -b)
    if fast_norm(a) > fast_norm(b):
        return reduce_vectors(b, a)
    if fast_norm(b) > fast_norm(np.add(b, a)):
        return reduce_vectors(a, np.add(b, a))
    if fast_norm(b) > fast_norm(np.subtract(b, a)):
        return reduce_vectors(a, np.subtract(b, a))
    return [a, b]


def get_factors(n):
    """Generate all factors of n."""
    for x in range(1, n + 1):
        if n % x == 0:
            yield x


def make_interface(
    film="",
    subs="",
    atol=1,
    ltol=0.05,
    max_area=500,
    max_area_ratio_tol=1.00,
    seperation=3.0,
    vacuum=8.0,
    apply_strain=False,
):
    """
    Use as main function for making interfaces/heterostructures.

    Return mismatch and other information as info dict.

    Args:
       film: top/film material.

       subs: substrate/bottom/fixed material.

       seperation: minimum seperation between two.

       vacuum: vacuum will be added on both sides.
       So 2*vacuum will be added.
    """
    z = ZSLGenerator(
        max_area_ratio_tol=max_area_ratio_tol,
        max_area=max_area,
        max_length_tol=ltol,
        max_angle_tol=atol,
    )
    film = fix_pbc(film.center_around_origin([0, 0, 0]))
    subs = fix_pbc(subs.center_around_origin([0, 0, 0]))
    matches = list(z(film.lattice_mat[:2], subs.lattice_mat[:2], lowest=True))
    info = {}
    info["mismatch_u"] = "na"
    info["mismatch_v"] = "na"
    info["mismatch_angle"] = "na"
    info["area1"] = "na"
    info["area2"] = "na"
    info["film_sl"] = "na"
    info["matches"] = matches
    info["subs_sl"] = "na"
    uv1 = matches[0]["sub_sl_vecs"]
    uv2 = matches[0]["film_sl_vecs"]
    u = np.array(uv1)
    v = np.array(uv2)
    a1 = u[0]
    a2 = u[1]
    b1 = v[0]
    b2 = v[1]
    mismatch_u = np.linalg.norm(b1) / np.linalg.norm(a1) - 1
    mismatch_v = np.linalg.norm(b2) / np.linalg.norm(a2) - 1
    angle1 = (
        np.arccos(np.dot(a1, a2) / np.linalg.norm(a1) / np.linalg.norm(a2))
        * 180
        / np.pi
    )
    angle2 = (
        np.arccos(np.dot(b1, b2) / np.linalg.norm(b1) / np.linalg.norm(b2))
        * 180
        / np.pi
    )
    mismatch_angle = abs(angle1 - angle2)
    area1 = np.linalg.norm(np.cross(a1, a2))
    area2 = np.linalg.norm(np.cross(b1, b2))
    uv_substrate = uv1
    uv_film = uv2
    substrate_latt = Lattice(
        np.array(
            [uv_substrate[0][:], uv_substrate[1][:], subs.lattice_mat[2, :]]
        )
    )
    _, __, scell = subs.lattice.find_matches(
        substrate_latt, ltol=ltol, atol=atol
    )
    film_latt = Lattice(
        np.array([uv_film[0][:], uv_film[1][:], film.lattice_mat[2, :]])
    )
    scell[2] = np.array([0, 0, 1])
    scell_subs = scell
    _, __, scell = film.lattice.find_matches(film_latt, ltol=ltol, atol=atol)
    scell[2] = np.array([0, 0, 1])
    scell_film = scell
    film_scell = film.make_supercell_matrix(scell_film)
    subs_scell = subs.make_supercell_matrix(scell_subs)
    info["mismatch_u"] = mismatch_u
    info["mismatch_v"] = mismatch_v
    print("mismatch_u,mismatch_v", mismatch_u, mismatch_v)
    info["mismatch_angle"] = mismatch_angle
    info["area1"] = area1
    info["area2"] = area2
    info["film_sl"] = film_scell
    info["subs_sl"] = subs_scell
    substrate_top_z = max(np.array(subs_scell.cart_coords)[:, 2])
    substrate_bot_z = min(np.array(subs_scell.cart_coords)[:, 2])
    film_top_z = max(np.array(film_scell.cart_coords)[:, 2])
    film_bottom_z = min(np.array(film_scell.cart_coords)[:, 2])
    thickness_sub = abs(substrate_top_z - substrate_bot_z)
    thickness_film = abs(film_top_z - film_bottom_z)
    sub_z = (
        (vacuum + substrate_top_z)
        * np.array(subs_scell.lattice_mat[2, :])
        / np.linalg.norm(subs_scell.lattice_mat[2, :])
    )
    shift_normal = (
        sub_z / np.linalg.norm(sub_z) * seperation / np.linalg.norm(sub_z)
    )
    tmp = (
        thickness_film / 2 + seperation + thickness_sub / 2
    ) / np.linalg.norm(subs_scell.lattice_mat[2, :])
    shift_normal = (
        tmp
        * np.array(subs_scell.lattice_mat[2, :])
        / np.linalg.norm(subs_scell.lattice_mat[2, :])
    )
    interface = add_atoms(
        film_scell, subs_scell, shift_normal, apply_strain=apply_strain
    ).center_around_origin([0, 0, 0.5])
    combined = interface.center(vacuum=vacuum).center_around_origin(
        [0, 0, 0.5]
    )
    info["interface"] = combined
    return info


def get_hetero_type(A={}, B={}):
    """Provide heterojunction classification using Anderson rule."""
    stack = "na"
    int_type = "na"
    try:
        if A["scf_vbm"] - A["avg_max"] < B["scf_vbm"] - B["avg_max"]:
            stack = "BA"
        else:
            C = A
            D = B
            A = D
            B = C
            stack = "AB"
        vbm_a = A["scf_vbm"] - A["avg_max"]
        vbm_b = B["scf_vbm"] - B["avg_max"]
        cbm_a = A["scf_cbm"] - A["avg_max"]
        cbm_b = B["scf_cbm"] - B["avg_max"]
        if vbm_a < vbm_b and vbm_b < cbm_b and cbm_b < cbm_a:
            int_type = "I"
        elif vbm_a < vbm_b and vbm_b < cbm_a and cbm_a < cbm_b:
            int_type = "II"
        elif vbm_a < cbm_a and cbm_a < vbm_b and vbm_b < cbm_b:
            int_type = "III"
    except Exception:
        pass
    return int_type, stack


def mismatch_strts():
    """
    Return mismatch and other information as info dict.

    Deprecated, preserved here so legacy imports work.
    """
    pass


def get_hetero():
    """Generate heterostructure. Deprecated also."""
    pass


"""
if __name__ == "__main__":
    s1 = Poscar.from_file(
        "MAIN-RELAX-Surf-mp-1821/POSCAR"
    )
    s2 = Poscar.from_file(
        "MAIN-RELAX-Surf-mp-2815/POSCAR"
    )
    print(s1)
    print(s2)
    info = mismatch_strts(film=s1.atoms, subs=s2.atoms)
    # print (info)
    print((s1.__dict__["atoms"].lattice_mat))
    d = s1.__dict__["atoms"]
    f = open("test.json", "wb")
    joblib.dump(s1, f)
    f.close()
    print(get_hetero(s1.atoms, s2.atoms))
    ff = open("test.json", "rb")
    dd = joblib.load(ff)
    ff.close()
    # print (dd.atoms.lattice_mat)
"""



from jarvis.analysis.thermodynamics.energetics import form_enp,get_twod_defect_energy
import os
from jarvis.io.vasp.outputs import Vasprun
tmp_xml=os.path.join(os.path.dirname(__file__), "JVASP-667_C_C_c.xml")
vrun=Vasprun(tmp_xml)
def test_get_twod_defect_energy():
  Ef=get_twod_defect_energy(vrun=vrun,jid="JVASP-667",atom="C")
  print (Ef)
def test_form_enp():
   atoms=vrun.all_structures[-1]
   total_energy=vrun.final_energy
   Ef=form_enp(atoms=atoms,total_energy=total_energy)
   print (Ef)
#test_get_twod_defect_energy()
#test_form_enp()

